# coding:utf8

from fastweb.accesspoint import ioloop
from fastweb.component.db.mysql import SyncMysql, AsynMysql
from fastweb.pool import SyncConnectionPool, AsynConnectionPool


setting = {'host': 'localhost', 'port': 3306, 'user': 'root', 'password': ''}


class TestSyncPool(object):
    def test_create(self):
        pool = SyncConnectionPool(SyncMysql, setting, name='test sync mysql pool', size=5, awake=10)
        pool.create()

    def test_add_connection(self):
        pool = SyncConnectionPool(SyncMysql, setting, name='test sync mysql pool', size=5, awake=10)
        pool.create()
        pool.add_connection()

    def test_get_connection(self):
        pool = SyncConnectionPool(SyncMysql, setting, name='test sync mysql pool', size=5, awake=10)
        pool.create()
        assert isinstance(pool.get_connection(), SyncMysql)

    def test_rescue(self):
        pool = SyncConnectionPool(SyncMysql, setting, name='test sync mysql pool', size=5, awake=10)
        pool.create()
        pool.rescue()

    def test_concurrency_get_connection(self):
        """大并发下的获取连接"""
        pass

    def test_maxconn_rescue(self):
        """动态扩展到最大连接数"""
        pass


class TestAsynPool(object):
    def test_create(self):
        pool = AsynConnectionPool(AsynMysql, setting, name='test asyn pool', size=5, awake=10)
        ioloop.IOLoop.current().run_sync(pool.create)

    def test_add_connection(self):
        pool = AsynConnectionPool(AsynMysql, setting, name='test asyn pool', size=5, awake=10)
        ioloop.IOLoop.current().run_sync(pool.create)
        ioloop.IOLoop.current().run_sync(pool.add_connection)

    def test_get_connection(self):
        pool = AsynConnectionPool(AsynMysql, setting, name='test asyn pool', size=5, awake=10)
        ioloop.IOLoop.current().run_sync(pool.create)
        assert isinstance(pool.get_connection(), AsynMysql)

    def test_rescue(self):
        pool = AsynConnectionPool(AsynMysql, setting, name='test asyn pool', size=5, awake=10)
        ioloop.IOLoop.current().run_sync(pool.create)
        ioloop.IOLoop.current().run_sync(pool.rescue)

    def test_concurrency_get_connection(self):
        """大并发下的获取连接"""
        pass

    def test_maxconn_rescue(self):
        """动态扩展到最大连接数"""
        pass





# coding:utf8

"""任务模块"""

from fastweb.spec import torcelery
from fastweb.util.tool import timing
from fastweb.component import Component
from fastweb.exception import TaskError
from fastweb.accesspoint import CeleryTask, Celery, Queue, Exchange, coroutine, Return


DEFAULT_TIMEOUT = 5


class Task(Component, CeleryTask):
    """任务类"""

    eattr = {'broker': str, 'queue': str, 'exchange': str, 'routing_key': str, 'backend': str}
    oattr = {'timeout': int}

    def __init__(self, setting):
        """初始化任务"""

        Component.__init__(self, setting)
        CeleryTask.__init__(self)

        # 设置任务的属性
        self.name = setting['_name']
        exchange_type = self.setting.get('exchange_type', 'direct')
        self.timeout = self.setting.get('timeout', DEFAULT_TIMEOUT)
        rate_limit = self.setting.get('rate_limit', None)
        acks_late = self.setting.get('acks_late', None)
        self.requestid = None

        # 设置绑定的application的属性
        app = Celery(main=self.name, broker=self.broker, backend=self.backend)
        app.tasks.register(self)
        self.backend = app.backend

        # 设置任务的路由
        queue = Queue(name=self.queue, exchange=Exchange(name=self.exchange, type=exchange_type), routing_key=self.routing_key)
        app.conf.update(task_queues=(queue,),
                        task_routes={self.name: {'queue': self.queue, 'routing_key': self.routing_key}},
                        task_annotations={self.name: {'rate_limit': rate_limit}},
                        task_acks_late=acks_late)

        # task和application绑定
        self.application = app
        self.bind(app)


class SyncTask(Task):
    """同步任务类"""

    def __str__(self):
        return '<SyncTask: {name} of queue({queue}) exchange({exchange}) routing_key({routing_key})>'.\
            format(name=self.name, queue=self.queue, exchange=self.exchange, routing_key=self.routing_key)

    def call_async(self, *args, **kwargs):
        """异步调用"""
        self.requestid = self.owner.requestid if self.owner else None
        with timing('ms', 10) as t:
            self.recorder('INFO', 'asynchronous call {obj} start'.format(obj=self))
            # 调用task层时，调用方的requestid会成为本次任务的taskid， 任务的requesid也为透传id
            taskid = self.apply_async(task_id=self.requestid,
                                      queue=self.queue,
                                      exchange=self.exchange,
                                      routing_key=self.routing_key,
                                      *args,
                                      **kwargs)
        self.recorder('INFO', 'asynchronous call {obj} successful -- {t}'.format(obj=self, t=t))
        return taskid

    def call(self, *args, **kwargs):
        """同步调用"""
        # TODO:多个同步任务的链式调用
        self.requestid = self.owner.requestid if self.owner else None
        with timing('ms', 10) as t:
            self.recorder('INFO', 'synchronize call {task} start'.format(task=self))
            # 调用task层时，调用方的requestid会成为本次任务的taskid， 任务的requesid也为透传id
            result = self.apply_async(task_id=self.requestid,
                                      queue=self.queue,
                                      exchange=self.exchange,
                                      routing_key=self.routing_key,
                                      *args,
                                      **kwargs)

            while True:
                if result.ready():
                    break

        if not result:
            self.recorder('ERROR',
                          'synchronize call {obj} timeout -- {t}'.format(obj=self, ret=result, t=t))
            raise TaskError

        self.recorder('INFO', 'synchronize call {obj} successful -- {ret} -- {t}'.format(obj=self,
                                                                                         ret=result,
                                                                                         t=t))
        return result.result


class AsynTask(Task):
    """异步任务类"""
    def __str__(self):
        return '<AsynTask: {name} of queue({queue}) exchange({exchange}) routing_key({routing_key})>'.\
            format(name=self.name, queue=self.queue, exchange=self.exchange, routing_key=self.routing_key)

    @coroutine
    def call_async(self, *args, **kwargs):
        """异步调用"""
        self.requestid = self.owner.requestid if self.owner else None
        with timing('ms', 10) as t:
            self.recorder('INFO', 'asynchronous call {obj} start\nArgument: {args} {kwargs}'.format(obj=self,
                                                                                                    args=args,
                                                                                                    kwargs=kwargs))
            # 调用task层时，调用方的requestid会成为本次任务的taskid， 任务的requesid也为透传id
            taskid = yield torcelery.async(self,
                                           task_id=self.requestid,
                                           queue=self.queue,
                                           exchange=self.exchange,
                                           routing_key=self.routing_key,
                                           *args,
                                           **kwargs)
        self.recorder('INFO', 'asynchronous call {obj} successful -- {t}'.format(obj=self, t=t))
        raise Return(taskid)

    @coroutine
    def call(self, *args, **kwargs):
        """同步调用"""
        self.requestid = self.owner.requestid if self.owner else None
        # TODO:多个同步任务的链式调用
        with timing('ms', 10) as t:
            self.recorder('INFO', 'synchronize call {obj} start\nArgument: {args} {kwargs}'.format(obj=self,
                                                                                                   args=args,
                                                                                                   kwargs=kwargs))
            # 调用task层时，调用方的requestid会成为本次任务的taskid， 任务的requesid也为透传id
            result = yield torcelery.sync(self, self.timeout,
                                          task_id=self.requestid,
                                          queue=self.queue,
                                          exchange=self.exchange,
                                          routing_key=self.routing_key,
                                          *args,
                                          **kwargs)

        if not result:
            self.recorder('ERROR',
                          'synchronize call {obj} timeout -- {t}'.format(obj=self, ret=result, t=t))
            raise TaskError

        self.recorder('INFO', 'synchronize call {obj} successful\n'
                              'Return:{ret} -- {t}'.format(obj=self,
                                                                                         ret=result,
                                                                                         t=t))
        raise Return(result)



# coding:utf8

"""网络层模块"""

import json
import shlex
import traceback
import subprocess


from fastweb.accesspoint import (web, coroutine, Task, Return, options,
                                 AsyncHTTPClient, HTTPError, Subprocess, httpserver, ioloop, run_on_executor)

from fastweb import app
import fastweb.components
from fastweb.util.tool import timing
from fastweb.util.thread import FThread
from fastweb.util.python import to_plain
from fastweb.util.log import recorder, console_recorder
from fastweb.exception import HttpError, SubProcessError


__all__ = ['Api', 'Page', 'arguments', 'start_web_server']


class AsynComponents(fastweb.components.Components):
    """异步组件类"""

    @coroutine
    def http_request(self, request):
        """http请求

        :parameter:
          - `request`:http请求
        """

        self.recorder(
            'INFO', 'http request start\n{request}'.format(request=request))

        with timing('ms', 10) as t:
            try:
                response = yield AsyncHTTPClient().fetch(request)
            except HTTPError as ex:
                self.recorder('ERROR', 'http request error {request} ({e})'.format(
                    request=request, e=ex))
                raise HttpError

        self.recorder('INFO', 'http request successful\t({response} <{time}>)'.format(
            response=response.code, time=t))
        raise Return(response)

    @coroutine
    def call_subprocess(self, command, stdin_data=None, stdin_async=True):
        """命令行调用

        :parameter:
          - `command`:命令行
          - `stdin_data`:传入数据
        """

        # TODO:待优化
        self.recorder(
            'INFO', 'call subprocess start <{cmd}>'.format(cmd=command))

        with timing('ms', 10) as t:
            stdin = Subprocess.STREAM if stdin_async else subprocess.PIPE
            sub_process = Subprocess(shlex.split(command),
                                     stdin=stdin,
                                     stdout=Subprocess.STREAM,
                                     stderr=Subprocess.STREAM)
            try:
                if stdin_data:
                    if stdin_async:
                        yield Task(sub_process.stdin.write, stdin_data)
                    else:
                        sub_process.stdin.write(stdin_data)

                if stdin_async or stdin_data:
                    sub_process.stdin.close()

                result, error = yield [Task(sub_process.stdout.read_until_close),
                                       Task(sub_process.stderr.read_until_close)]
            except (OSError, ValueError) as ex:
                self.recorder('ERROR', 'call subprocess <{cmd} <{time}> {e} {msg}'.format(
                    cmd=command, time=t, e=ex, msg=result.strip() if result else error.strip()))
                raise SubProcessError

        if sub_process.returncode != 0:
            self.recorder('ERROR', 'call subprocess <{cmd}> <{time}> {msg}'.format(
                cmd=command, time=t, msg=result.strip() if result else error.strip()))
            raise SubProcessError

        self.recorder('INFO', 'call subprocess <{cmd}> <{time} {msg}>'.format(
            cmd=command, time=t, msg=result.strip() if result else error.strip()))
        raise Return((result, error))

    def call_celery(self, *args, **kwargs):
        pass


class Api(web.RequestHandler, AsynComponents):
    """Api操作基类"""

    def __init__(self, application, request, **kwargs):
        super(Api, self).__init__(application, request, **kwargs)

        self.uri = request.uri
        self.host = request.host
        self.remoteip = request.remote_ip
        self.arguments = self.request.arguments
        self.requestid = self.get_argument('requestid') if self.get_argument('requestid', None) else self.gen_requestid()

        # TODO: 远程ip获取不准确
        self.recorder(
            'IMPORTANT',
            'Api request\nIp:<{ip}>\nHost:<{host}{uri}\nArguments:<{arguments}>\nUserAgent:<{ua}>'.format(
                ip=self.remoteip,
                host=self.host,
                uri=self.uri,
                arguments=self.request.body,
                ua=self.request.headers['User-Agent']))
        self.set_header_json()

    def data_received(self, chunk):
        pass

    def log_exception(self, typ, value, tb):
        """日志记录异常,并自动返回系统错误"""

        console_recorder('ERROR', '{message}'.format(
            message=traceback.format_exc()))
        self.recorder('ERROR', '{message}'.format(
            message=traceback.format_exc()))
        self.end('SVR')

    def set_ajax_cors(self, allow_ip):
        """设置CORS"""

        header = 'Access-Control-Allow-Origin'
        self.set_header(header, allow_ip)
        self.recorder('INFO', 'set header <{key}:{ip}>'.format(
            key=header, ip=allow_ip))

    def set_header_json(self):
        """设置返回格式为json"""

        header = 'Content-type'
        self.add_header(header, 'text/json')
        self.recorder('INFO', 'set header <{key}:{type}>'.format(
            key=header, type='text/json'))

    def end(self, code='SUC', log=True, **kwargs):
        """请求结束"""

        ret = self.errcode[code]
        ret = dict(ret, **kwargs)
        self.write(json.dumps(ret))
        self.finish()
        self.release()
        t = (self.request._finish_time-self.request._start_time) * 1000

        if log:
            self.recorder(
                'IMPORTANT',
                'Api response\nResponse:<{ret}>\nTime:<{time}ms>'.format(
                    ret=ret, time=t))
        else:
            self.recorder(
                'IMPORTANT',
                'Api response\nTime:<{time}ms>'.format(time=t))


class Page(web.RequestHandler, AsynComponents):
    """Page操作基类"""

    def __init__(self, application, request, **kwargs):
        super(Page, self).__init__(application, request, **kwargs)

        self.uri = request.uri
        self.host = request.host
        self.remoteip = request.remote_ip
        self.arguments = self.request.arguments
        self.requestid = self.get_argument('requestid') if self.get_argument('requestid', None) else self.gen_requestid()

        self.recorder(
            'IMPORTANT',
            'Page request\nIp:<{ip}>\nHost:<{host}{uri}\nArguments:<{arguments}>\nUserAgent:<{ua}>'.format(
                ip=self.remoteip,
                host=self.host,
                uri=self.uri,
                arguments=self.request.body,
                ua=self.request.headers['User-Agent']))

    def data_received(self, chunk):
        pass

    def log_exception(self, typ, value, tb):
        """日志记录异常"""

        console_recorder('ERROR', '{message}'.format(
            message=traceback.format_exc()))
        self.recorder('ERROR', '{message}'.format(
            message=traceback.format_exc()))

    def end(self, template=None, log=True, **kwargs):
        """ 请求结束"""

        if template:
            # TODO:模版位置的正确性
            self.render(template, **kwargs)

        self.release()
        t = (self.request._finish_time - self.request._start_time) * 1000

        if log:
            self.recorder(
                'IMPORTANT',
                'Page response\nTemplate:{tem}\nTemArgs:{args}\nTime:<{time}ms>'.format(
                    tem=template, args=kwargs, time=t))
        else:
            self.recorder(
                'IMPORTANT',
                'Page response\nTemplate:{tem}\nTime:<{time}ms>'.format(
                    tem=template, time=t))


def arguments(convert=None, **ckargs):
    """检查并转换请求参数是否合法并转换参数类型

    :parameter:
      - `convert`:待转换的key,但非必要参数
      - `**ckargs`:必要参数
    """

    def _deco(fn):
        def _wrap(cls, *args, **kwargs):
            if convert:
                for cname, ctype in convert.items():
                    cvalue = cls.request.arguments.get(cname)
                    cvalue = to_plain(cvalue)
                    if cvalue:
                        cls.request.arguments[cname] = ctype(cvalue)

            for cname, ctype in ckargs.items():
                cvalue = cls.request.arguments.get(cname)
                cvalue = to_plain(cvalue)

                def invalid_recorder(msg):
                    diff = set(cls.request.arguments.keys()).symmetric_difference(set(ckargs.keys()))
                    cls.recorder('error', 'check arguments invalid <{diff}> {msg}'.format(
                        msg=msg, diff=to_plain(diff)))
                    cls.end('SVR')

                if cvalue:
                    if ctype is int:
                        if not cvalue.isdigit():
                            invalid_recorder('argument type error')
                            return
                    elif not isinstance(cvalue, ctype):
                        invalid_recorder('argument type error')
                        return
                else:
                    if isinstance(cls, Api):
                        invalid_recorder('argument empty')
                        return
                    elif isinstance(cls, Page):
                        invalid_recorder('argument empty')
                        return
                cls.request.arguments[cname] = ctype(cvalue)
            return fn(cls, *args, **kwargs)
        return _wrap
    return _deco


def start_web_server(port, handlers, **settings):
    """启动服务器"""
    if not app.bRecorder:
        app.load_recorder()

    application = web.Application(
        handlers,
        **settings
    )

    http_server = httpserver.HTTPServer(
        application, xheaders=settings.get('xheaders'))
    http_server.listen(port)
    recorder('INFO', 'server start on {port}'.format(port=port))
    try:
        ioloop.IOLoop.current().start()
    except KeyboardInterrupt:
        ioloop.IOLoop.current().stop()
        FThread.stop(0)
        recorder('INFO', 'server stop on {port}'.format(port=port))


def set_error_handler():
    # TODO:设置错误的句柄
    pass





# coding:utf8

import unittest

from fastweb import ioloop
from fastweb import coroutine
from fastweb.loader import app
from fastweb.component.db.mongo import SyncMongo, AsynMongo


app.load_recorder('log/app.log', system_log_path='log/sys.log', system_level='DEBUG')


class SyncMongoTest(unittest.TestCase):

    def runTest(self):

        setting = {'host': 'localhost', 'db': 'local'}
        mongo = SyncMongo(setting).set_name('mongo_test')
        mongo.connect()

        mongo.query({'find': 'startup_log'})
        mongo.print_response()

        # print(mongo.query('show dbs'))


class AsynMongoTest(unittest.TestCase):

    @coroutine
    def q(self):
        setting = {'host': 'localhost', 'db': 'local', 'port': 27017}
        self.mongo = AsynMongo(setting).set_name('mongo_test')
        yield self.mongo.connect()
        yield self.mongo.query({'find': 'startup_log'})
        self.mongo.print_response()

    def runTest(self):
        ioloop.IOLoop.instance().run_sync(self.q)


if __name__ == '__main__':
    unittest.main()



# coding:utf8

import pprint
import pymongo
from pymongo import MongoClient
from motor import motor_tornado
from tornado.gen import coroutine, Return
from pymongo.errors import ConnectionFailure

import fastweb.util.tool as tool
from fastweb.util.python import dumps
from fastweb.exception import MongoError
from fastweb.component import Component


DEFAULT_PORT = 27017
DEFAULT_TIMEOUT = 5


class Mongo(Component):
    """Mongo组件基类"""

    eattr = {'host': str}
    oattr = {'port': int, 'user': str, 'password': str, 'db': str, 'timeout': int}

    def __init__(self, setting):
        self.port = DEFAULT_PORT
        self.timeout = DEFAULT_TIMEOUT

        super(Mongo, self).__init__(setting)

        self.isConnect = False
        self._client = None
        self._db = None
        self._response = None

        self._prepare()

    def _prepare(self):
        self.host = 'mongodb://{user}:{password}@{host}'.format(user=self.user, password=self.password, host=self.host) if self.user and self.password else 'mongodb://{host}'.format(host=self.host)
        self.setting['host'] = self.host
        self.setting.pop('db', None)
        self.setting.pop('user', None)
        self.setting.pop('password', None)
        self.setting['connectTimeoutMS'] = self.setting.pop('timeout', None)

    def connect(self):
        raise NotImplementedError

    def select_db(self, db):
        """选择数据库"""

        try:
            self.recorder('INFO', '{obj} select database [{db}]'.format(obj=self, db=db))
            self._db = getattr(self._client, db)
        except AttributeError as e:
            self.recorder('ERROR', '{obj} select database error [msg]'.format(obj=self, msg=e))
            raise MongoError

    def query(self):
        raise NotImplementedError

    @staticmethod
    def _parse_response(response):
        """解析response
           结果为空时fisrtBatch中为空列表
           TODO:不知道什么时候ok返回非1值"""

        if response['ok']:
            return response['cursor']['firstBatch']
        return response

    def print_response(self):
        """打印返回值,调试使用"""

        pprint.pprint(self._response)

    def error(self):
        self.isConnect = False

    def reconnect(self):
        pass

    def ping(self):
        pass


class SyncMongo(Mongo):
    """"同步mongo
        线程不安全
    """

    def __reduce__(self):
        return SyncMongo, (self.setting,)

    def __str__(self):
        return '<SyncMongo {name} {host} {port} {db}>'.format(
            name=self.name, host=self.host, port=self.port, db=self.db)

    def connect(self):
        """建立连接"""

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self.set_idle()
            self._client = MongoClient(**self.setting)
            if self.db:
                self.select_db(self.db)
            self.isConnect = True
            self.recorder('INFO', '{obj} connect successful'.format(obj=self))
        except ConnectionFailure as e:
            self.recorder('ERROR', '{obj} connect failed [{msg}]'.format(obj=self, msg=e))
            self.error()
            raise MongoError
        return self

    def query(self, command, value=1, check=True, allowable_errors=None, **kwargs):
        """命令行操作
           pymongo内部没有实现重试,需要进行重试
           TODO:对不同的异常做不同的操作"""

        if not self._db:
            self.recorder('CRITICAL', 'please select db first!')

        shell_command = 'db.runCommand(\n{cmd}\n)'.format(cmd=dumps(command, indent=4, whole=4))
        self.recorder('INFO', '{obj} command start\n{cmd}'.format(obj=self, cmd=shell_command))
        try:
            with tool.timing('s', 10) as t:
                response = self._db.command(command=command, value=value, check=check, allowable_errors=allowable_errors, **kwargs)
        except pymongo.errors.PyMongoError as e:
            self.recorder('ERROR', '{obj} command error [{msg}]'.format(obj=self, msg=e))
            raise MongoError
        self.recorder('INFO', '{obj} command successful\n{cmd} -- {time}'.format(obj=self, cmd=shell_command, time=t))

        self._response = self._parse_response(response)
        return self._response


class AsynMongo(Mongo):
    """异步Mongo组件"""

    def __reduce__(self):
        return AsynMongo, (self.setting,)

    def __str__(self):
        return '<AsynMongo {host} {port} {name}>'.format(
            host=self.host, port=self.port, name=self.name)

    @coroutine
    def connect(self):
        """建立连接"""

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self.set_idle()
            self._client = motor_tornado.MotorClient(**self.setting)
            if self.db:
                self.select_db(self.db)
            self.isConnect = True
            self.recorder('INFO', '{obj} connect successful'.format(obj=self))
        except ConnectionFailure as e:
            self.recorder('ERROR', '{obj} connect failed [{msg}]'.format(obj=self, msg=e))
            self.error()
            raise MongoError
        raise Return(self)

    @coroutine
    def query(self, command, value=1, check=True, allowable_errors=None, **kwargs):
        """命令行操作
           pymongo内部没有实现重试,需要进行重试
           TODO:对不同的异常做不同的操作"""

        if not self._db:
            self.recorder('CRITICAL', 'please select db first!')

        shell_command = 'db.runCommand(\n{cmd}\n)'.format(cmd=dumps(command, indent=4, whole=4))
        self.recorder('INFO', '{obj} command start\n{cmd}'.format(obj=self, cmd=shell_command))
        try:
            with tool.timing('s', 10) as t:
                response = yield self._db.command(command=command, value=value, check=check,
                                                  allowable_errors=allowable_errors, **kwargs)
        except pymongo.errors.PyMongoError as e:
            self.recorder('ERROR', '{obj} command error [{msg}]'.format(obj=self, msg=e))
            raise MongoError
        self.recorder('INFO', '{obj} command successful\n{cmd} -- {time}'.format(obj=self, cmd=shell_command, time=t))

        self._response = self._parse_response(response)
        raise Return(self._response)



# coding:utf8

"""消费者模块"""

import os
import sys
import json
from multiprocessing import Process

from fastweb import app
from fastweb.manager import Manager
from fastweb.util.tool import timing
from fastweb.exception import TaskError
from fastweb.component.task import Task
from fastweb.util.python import load_object
from fastweb.components import SyncComponents
from fastweb.accesspoint import CeleryTask


__all__ = ['start_task_worker']
DEFAULT_TIMEOUT = 5


class IFaceWorker(CeleryTask):
    def on_success(self, retval, task_id, args, kwargs):
        pass

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        pass

    def on_retry(self, exc, task_id, args, kwargs, einfo):
        pass

    def after_return(self, status, retval, task_id, args, kwargs, einfo):
        pass


class Worker(Task):
    """工作者类"""

    eattr = {'task_class': str, 'broker': str, 'queue': str, 'exchange': str, 'routing_key': str, 'backend': str}
    oattr = {'timeout': int, 'rate_limit': str, 'acks_late': bool}

    def __init__(self, setting):
        """初始化任务"""

        super(Worker, self).__init__(setting)

        import sys
        sys.path.append(os.getcwd())
        del sys

        # 设置执行任务的类
        self._task_cls = load_object(self.task_class)
        self._worker_obj = None

    def __str__(self):
        return '<Task: {name} of queue({queue}) exchange({exchange}) routing_key({routing_key})>'.\
            format(name=self.name, queue=self.queue, exchange=self.exchange, routing_key=self.routing_key)

    def run(self, *args, **kwargs):
        """任务处理
        转发给具体执行对象的run方法"""

        # Components中生成requestid
        self._worker_obj = type('Worker', (self._task_cls, SyncComponents, IFaceWorker), {})()
        self._worker_obj.requestid = self.request.id

        if hasattr(self._worker_obj, 'run'):
            self._worker_obj.recorder('IMPORTANT', '{obj} start\nRequest:\n{request}\nArgument: {args}\t{kwargs}'.format(obj=self,
                                                                                                                         request=json.dumps(self.request.as_execution_options(), indent=4),
                                                                                                                         args=args,
                                                                                                                         kwargs=kwargs))
            with timing('ms', 10) as t:
                ret = self._worker_obj.run(*args, **kwargs)
            self._worker_obj.recorder('IMPORTANT', '{obj} end\nReturn: {r} -- {t}'.format(obj=self, r=ret, t=t))
            return ret
        else:
            self._worker_obj.recorder('CRITICAL', '{obj} must have run function!'.format(obj=self))
            raise TaskError

    def on_success(self, retval, task_id, args, kwargs):
        """
        任务成功回调函数转发

        Run by the worker if the task executes successfully.

        Arguments:
            retval (Any): The return value of the task.
            task_id (str): Unique id of the executed task.
            args (Tuple): Original arguments for the executed task.
            kwargs (Dict): Original keyword arguments for the executed task.

        Returns:
            None: The return value of this handler is ignored.
        """

        if hasattr(self._worker_obj, 'on_success'):
            self._worker_obj.recorder('INFO', '{obj} success callback start'.format(obj=self))
            with timing('ms', 10) as t:
                r = self._worker_obj.on_success(retval, task_id, args, kwargs)
            self._worker_obj.recorder('INFO', '{obj} success callback end -- {t}'.format(obj=self, t=t))
            return r

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        """
        任务失败回调函数转发

        This is run by the worker when the task fails.

        Arguments:
            exc (Exception): The exception raised by the task.
            task_id (str): Unique id of the failed task.
            args (Tuple): Original arguments for the task that failed.
            kwargs (Dict): Original keyword arguments for the task that failed.
            einfo (~billiard.einfo.ExceptionInfo): Exception information.

        Returns:
            None: The return value of this handler is ignored.
        """

        self._worker_obj.recorder('ERROR', '{obj} failure\nTaskid: {id}\nException: {e}'.format(obj=self,
                                                                                                id=task_id,
                                                                                                e=exc))

        if hasattr(self._worker_obj, 'on_failure'):
            self._worker_obj.recorder('INFO', '{obj} failure callback start'.format(obj=self))
            with timing('ms', 10) as t:
                r = self._worker_obj.on_failure(exc, task_id, args, kwargs, einfo)
            self._worker_obj.recorder('INFO', '{obj} failure callback end -- {t}'.format(obj=self, t=t))
            return r

    def on_retry(self, exc, task_id, args, kwargs, einfo):
        """任务重试回调函数转发

        This is run by the worker when the task is to be retried.

        Arguments:
            exc (Exception): The exception sent to :meth:`retry`.
            task_id (str): Unique id of the retried task.
            args (Tuple): Original arguments for the retried task.
            kwargs (Dict): Original keyword arguments for the retried task.
            einfo (~billiard.einfo.ExceptionInfo): Exception information.

        Returns:
            None: The return value of this handler is ignored.
        """

        if hasattr(self._worker_obj, 'on_retry'):
            self._worker_obj.recorder('INFO', '{obj} retry callback start'.format(obj=self))
            with timing('ms', 10) as t:
                self._worker_obj.on_retry(exc, task_id, args, kwargs, einfo)
            self._worker_obj.recorder('INFO', '{obj} retry callback end -- {t}'.format(obj=self, t=t))
        else:
            self._worker_obj.recorder('ERROR', '{obj} retry\nTaskid: {id}\nException: {e}'.format(obj=self,
                                                                                                  id=task_id,
                                                                                                  e=exc))

    def after_return(self, status, retval, task_id, args, kwargs, einfo):
        """任务返回回调函数转发

        Arguments:
            status (str): Current task state.
            retval (Any): Task return value/exception.
            task_id (str): Unique id of the task.
            args (Tuple): Original arguments for the task.
            kwargs (Dict): Original keyword arguments for the task.
            einfo (~billiard.einfo.ExceptionInfo): Exception information.

        Returns:
            None: The return value of this handler is ignored.
        """

        if hasattr(self._worker_obj, 'after_return'):
            self._worker_obj.recorder('INFO', '{obj} after return callback start'.format(obj=self))
            with timing('ms', 10) as t:
                self._worker_obj.after_return(status, retval, task_id, args, kwargs, einfo)
            self._worker_obj.recorder('INFO', '{obj} after return callback end -- {t}'.format(obj=self, t=t))

        self._worker_obj.release()


def start_task_worker():
    """启动任务消费者

    每个application在一个进程中，不推荐定义大于CPU核数个application"""

    if not app.bRecorder:
        app.load_recorder()

    # 通过篡改命令行的参数更改application的node名称
    # 命令行中的-n参数会失效

    tasks = Manager.get_classified_components('worker')

    for task in tasks:
        argv = sys.argv
        argv.append('-n')
        argv.append('fastweb@celery@{app}'.format(app=task.name))
        p = Process(target=task.application.start, args=(argv,))
        p.start()



# coding;utf8


from fastweb.accesspoint import ioloop, coroutine
from fastweb.component.db.mysql import SyncMysql, AsynMysql


setting = {'host': 'localhost', 'port': 3306, 'user': 'root', 'password': ''}


class TestSyncMysql(object):
    def test_connect(self):
        mysql = SyncMysql(setting).set_name('sync_mysql_test')
        mysql.connect()

    def test_query(self):
        mysql = SyncMysql(setting).set_name('sync_mysql_test')
        mysql.connect()
        assert mysql.query('select * from mysql.user;')
        assert mysql.fetch()


class TestAsynMysql(object):
    def test_connect(self):
        mysql = AsynMysql(setting).set_name('asyn_mysql_test')
        ioloop.IOLoop.instance().run_sync(mysql.connect)

    def test_query(self):
        mysql = AsynMysql(setting).set_name('asyn_mysql_test')
        ioloop.IOLoop.instance().run_sync(mysql.connect)

        @coroutine
        def _query():
            ret = yield mysql.query('select * from mysql.user;')
            assert ret

        ioloop.IOLoop.instance().run_sync(_query)



# coding:utf8

from fastweb import app
from fastweb.manager import Manager, SyncConnManager, AsynConnManager


class TestManager(object):

    def test_setup(self):
        configer = app.load_component('web', backend='ini', path='fastweb/test/config/service.ini')
        Manager.setup('web', configer)


class TestSyncManager(object):

    def test_setup(self):
        configer = app.load_component('web', backend='ini', path='fastweb/test/config/service.ini')
        SyncConnManager.setup(configer)


class TestAsynManager(object):

    def test_setup(self):
        configer = app.load_component('web', backend='ini', path='fastweb/test/config/service.ini')
        AsynConnManager.setup(configer)





# coding:utf8

"""组件工具模块"""

import types
import shlex
from concurrent.futures import ThreadPoolExecutor

import fastweb.manager
from fastweb import app
from fastweb.compat import subprocess
from fastweb.util.log import record, recorder
from fastweb.util.tool import uniqueid, timing, RetryPolicy, Retry
from fastweb.accesspoint import HTTPClient, CachingClient, UsernameToken, Error, Transport
from fastweb.exception import ComponentError, SubProcessError, SubProcessTimeoutError, HttpError, SoapError


__all__ = ['UsernameToken']


class Components(object):
    """组件基类
    基础组件功能
    """

    _blacklist = ['requestid', '_new_cookie', 'include_host',
                  '_active_modules', '_current_user', '_locale',
                  '__header__', 'priority', 'delivery_mode', 'compression', 'immediate', 'mandatory']
    executor = None

    def __init__(self):
        self.loader = fastweb.loader.app
        self.errcode = fastweb.loader.app.errcode
        self.configs = fastweb.loader.app.configs
        self.requestid = self.gen_requestid()

        # 组件缓冲池,确保同一请求对同一组件只获取一次
        self._components = {}

    def __getattr__(self, name):
        """获取组件

        :parameter:
          - `name`: 组件名称
        """

        if name in self._blacklist:
            recorder('WARN', '{attr} in blacklist'.format(attr=name))
            raise AttributeError

        # 缓冲池中存在则使用缓冲池中的组件
        component = self._components.get(name)

        if not component:
            component = fastweb.manager.Manager.get_component(name, self)

            if not component:
                self.recorder('ERROR', "can't acquire idle component <{name}>".format(name=name))
                raise ComponentError

            self._components[name] = component
            self.recorder('DEBUG', '{obj} get component from manager {name} {com}'.format(obj=self, name=name, com=component))
            return component
        else:
            self.recorder('DEBUG', '{obj} get component from components cache {name} {com}'.format(obj=self, name=name, com=component))
            return component

    @staticmethod
    def gen_requestid():
        """生成requestid"""

        return str(uniqueid())

    def load_executor(self, size):
        """加载当前handler级别的线程池"""

        self.executor = ThreadPoolExecutor(size)

    def add_blacklist(self, attr):
        """增加类属性黑名单

        :parameter:
          - `attr`:属性名称
        """

        self._blacklist.append(attr)

    def add_function(self, **kwargs):
        """增加方法到对象中"""

        # TODO:有没有更好的加载方式
        for callname, func in kwargs.items():
            setattr(self, '{callname}'.format(callname=callname), types.MethodType(func, self))

    def recorder(self, level, msg):
        """日志记录

        :parameter:
          - `level`:日志登记
          - `msg`:记录信息
        """

        record(level, msg, app.application_recorder, extra={'requestid': self.requestid})

    def release(self):
        """释放组件"""

        for name, component in self._components.items():
            fastweb.manager.Manager.return_component(name, component)
            self.recorder('DEBUG', '{com} return manager'.format(com=component))

        self._components.clear()
        self.recorder('INFO', 'release all used components')


class SyncComponents(Components):
    """同步组件类"""
    
    def __init__(self):
        super(SyncComponents, self).__init__()

    def http_request(self, request, timeout=None):
        http_retry_policy = RetryPolicy(times=request.retry, error=HttpError)
        request.request_timeout = timeout
        return Retry(self, '{obj}'.format(obj=self), self._http_request, http_retry_policy, request).run_sync()

    def _http_request(self, retry, request):
        """http请求

        :parameter:
          - `request`:http请求
        """

        if hasattr(self, 'requestid'):
            _recorder = self.recorder
        else:
            _recorder = recorder

        _recorder('INFO', 'http request start {request}'.format(request=request))

        with timing('ms', 10) as t:
            try:
                response = HTTPClient().fetch(request)
            except HttpError as ex:
                _recorder('ERROR', 'http request error {request} {e}'.format(request=request, e=ex))
                raise retry

        _recorder('INFO', 'http request successful\n{response} -- {time}'.format(response=response.code, time=t))
        return response

    def call_subprocess(self, command, stdin_data=None, timeout=None):
        """命令行调用

        :parameter:
          - `command`:命令行
          - `stdin_data`:传入数据
        """

        if hasattr(self, 'requestid'):
            _recorder = self.recorder
        else:
            _recorder = recorder

        _recorder('INFO', 'call subprocess start\n{cmd}'.format(cmd=command))

        with timing('ms', 10) as t:
            sub_process = subprocess.Popen(shlex.split(command),
                                           stdout=subprocess.PIPE,
                                           stderr=subprocess.PIPE)
            try:
                result, error = sub_process.communicate(stdin_data, timeout=timeout)
            except (OSError, ValueError) as ex:
                _recorder('ERROR', 'call subprocess\n({cmd}) ({e}) '.format(
                    cmd=command, e=ex, msg=result.strip() if result else error.strip()))
                raise SubProcessError
            except subprocess.TimeoutExpired:
                _recorder('ERROR', 'call subprocess timeout [{timeout}]'.format(timeout=timeout))
                raise SubProcessTimeoutError
            finally:
                sub_process.kill()

            if sub_process.returncode != 0:
                _recorder('ERROR', 'call subprocess error ({cmd}) <{time}> {msg}'.format(
                          cmd=command, time=t, msg=result.strip() if result else error.strip()))
                raise SubProcessError

        _recorder('INFO', 'call subprocess successful\n{cmd}\n{msg}\n<{time}>'.format(
            cmd=command, time=t, msg=result.strip() if result else error.strip()))
        return result, error

    def soap_request(self, request):
        """soap 请求"""

        soap_retry_policy = RetryPolicy(times=request.retry, error=SoapError)
        return Retry(self, '{obj}'.format(obj=self), self._soap_request, soap_retry_policy, request).run_sync()

    def _soap_request(self, retry, request):
        if hasattr(self, 'requestid'):
            _recorder = self.recorder
        else:
            _recorder = recorder

        _recorder('INFO', 'soap request start {request}'.format(request=request))

        with timing('ms', 10) as t:
            try:
                transport = Transport(request.timeout)
                client = CachingClient(wsdl=request.wsdl, wsse=request.wsse, transport=transport)
                response = getattr(client.service, request.function)(**request.kwargs)
            except Error as e:
                _recorder('ERROR', 'soap request error ({e})'.format(e=e))
                raise retry

        _recorder('INFO', 'soap request successful\n{response} -- {time}'.format(response=response, time=t))
        return response





# coding:utf8

"""tornado调用celery模块"""

from fastweb.accesspoint import states, IOLoop, TracebackFuture


def async(task, *args, **kwargs):
    """celery异步任务

    异步给celery发送命令时，任务的状态为PENDING则视为成功，将任务的taskid返回"""

    future = TracebackFuture()
    callback = kwargs.pop("callback", None)
    if callback:
        IOLoop.instance().add_future(future, lambda f: callback(f.result()))
    result = task.apply_async(*args, **kwargs)
    result.start = IOLoop.instance().time()
    IOLoop.instance().add_callback(_on_async_result, result, future)
    return future


def sync(task, timeout, *args, **kwargs):
    """celery同步任务

    同步给celery发送命令时，任务状态为READY则视为成功，将任务执行的结果返回

    :parameter:
      - `timeout`: 超时时间
    """

    future = TracebackFuture()
    callback = kwargs.pop("callback", None)
    if callback:
        IOLoop.instance().add_future(future, lambda f: callback(f.result()))
    result = task.apply_async(*args, **kwargs)
    IOLoop.instance().add_callback(_on_sync_result, result, future, IOLoop.instance().time(), timeout)
    return future


def _on_async_result(result, future):
    if result.state == states.PENDING:
        future.set_result(result.task_id)
    else:
        IOLoop.instance().add_callback(_on_async_result, result, future)


def _on_sync_result(result, future, start_time, timeout):
    if IOLoop.instance().time()-start_time < timeout:
        if result.successful():
            future.set_result(result.result)
        else:
            IOLoop.instance().add_callback(_on_sync_result, result, future, start_time, timeout)
    else:
        print 'xxxx'
        future.set_result(None)



# coding:utf8

"""兼容模块"""

import os
import sys


if sys.version_info < (3, 0):
    import ConfigParser as cParser
else:
    import configparser as cParser

if os.name == 'posix' and sys.version_info < (3, 0):
    import subprocess32 as subprocess
else:
    import subprocess






# coding:utf8

"""组件管理模块"""

import json
from collections import defaultdict

import fastweb.loader
from fastweb.util.log import recorder
from accesspoint import coroutine
from fastweb.exception import ManagerError
from fastweb.pool import ConnectionPool, SyncConnectionPool, AsynConnectionPool


class Manager(object):
    """管理器

    fastweb.setting.default_component.COMPONENTS
    fastweb.setting.default_connection_component.SYNC_CONN_COMPONENTS
    fastweb.setting.default_connection_component.ASYN_CONN_COMPONENTS
    存储组件名称及相应的组件类

    _pools存储所有的组件
    有多种不同的组件，可以复用的组件，在组件管理池中只有一个；不可以复用的组件，组件管理池中需要创建多个，并且存在组件状态
    在取出时判断取出的方式
    """

    # 组件池 _pools: {component_name: component_pool}
    # 被分类的组件池 _classified_pools: {cpre: [obj, obj, ..]}
    _pools = {}
    _classified_pools = defaultdict(list)

    @staticmethod
    def setup(layout, configer):
        """安装组件"""

        recorder('DEBUG', 'default component manager setup start')
        from fastweb.setting.default_component import COMPONENTS

        if layout in ['web']:
            from fastweb.setting.default_component import ASYN_COMPONENTS
            components = COMPONENTS + ASYN_COMPONENTS
        elif layout in ['service', 'task']:
            from fastweb.setting.default_component import SYNC_COMPONENTS
            components = COMPONENTS + SYNC_COMPONENTS
        else:
            recorder('ERROR', 'layout error {layout}'.format(layout=layout))
            raise ManagerError

        if configer:
            for (cpre, cls) in components:
                components = configer.get_components(cpre)

                for name, value in components.items():
                    config = configer.configs[name]
                    config['_name'] = value['object']
                    com = cls(config)
                    cls.name = value['object']
                    Manager._pools[value['object']] = com
                    Manager._classified_pools[cpre].append(com)

        recorder('DEBUG', 'manager setup successful\n{pool}'.format(pool=Manager._pools))

    @staticmethod
    def get_classified_components(cpre):
        """获取被分类的组件"""

        return Manager._classified_pools.get(cpre, [])

    @staticmethod
    def get_component(name, owner):
        """通过manager获取组件

        ManagerError:可能是配置文件错误或者程序错误,应该尽快进行处理,不应该再向下继续运行

        :parameter:
          - `name`:组件名称"""

        pool = Manager._pools.get(name)

        if pool:
            if isinstance(pool, ConnectionPool):
                component = pool.get_connection()
            else:
                component = pool
            component.set_used(owner)
            return component
        else:
            recorder('CRITICAL',
                     'get component ({name}) error,please check configuration\n{conf}'.format(conf=json.dumps(fastweb.loader.app.configs), name=name))
            raise ManagerError

    @staticmethod
    def return_component(name, component):
        """归还组件

        :parameter:
          - `name`:组件名称
          - `component`:组件
        """

        pool = Manager._pools.get(name)

        if pool:
            if isinstance(pool, ConnectionPool):
                pool.return_connection(component)
                component.set_idle()
        else:
            recorder('CRITICAL',
                     'please check configuration\n{conf}\n{name}'.format(conf=json.dumps(fastweb.loader.app.configs),
                                                                         name=name))
            raise ManagerError


class SyncConnManager(Manager):
    @staticmethod
    def setup(configer):
        """同步安装组件

         初始化组件时,尽快的抛出准确的错误信息
         """

        recorder('DEBUG', 'synchronize connection component manager setup start')
        from fastweb.setting.default_connection_component import SYNC_CONN_COMPONENTS

        if configer:
            for (cpre, cls, default_size) in SYNC_CONN_COMPONENTS:
                components = configer.get_components(cpre)

                for name, value in components.items():
                    config = configer.configs[name]
                    size = config.get('size', default_size)
                    awake = config.get('awake')
                    maxconnections = config.get('maxconnections')
                    pool = SyncConnectionPool(cls, config, size, name, awake=awake, maxconnections=maxconnections)
                    pool.create()
                    Manager._pools[value['object']] = pool

        recorder('DEBUG', 'synchronize manager setup successful')


class AsynConnManager(Manager):
    configer = None

    @staticmethod
    @coroutine
    def setup():
        """异步安装组件

        初始化组件时,尽快的抛出准确的错误信息
        """

        recorder('DEBUG', 'asynchronous connection component manager setup start')
        from fastweb.setting.default_connection_component import ASYN_CONN_COMPONENTS

        if AsynConnManager.configer:
            for (cpre, cls, default_size) in ASYN_CONN_COMPONENTS:
                components = AsynConnManager.configer.get_components(cpre)

                for name, value in components.items():
                    config = AsynConnManager.configer.configs[name]
                    size = config.get('size', default_size)
                    awake = config.get('awake')
                    maxconnections = config.get('maxconnections')
                    pool = AsynConnectionPool(cls, config, size, name, awake=awake, maxconnections=maxconnections)
                    yield pool.create()
                    Manager._pools[value['object']] = pool

        recorder('DEBUG', 'asynchronous manager setup successful')







# coding:utf8

import time


class Add(object):

    def run(self, x, y):
        print 'worker'
        print '啊啊啊啊啊啊啊啊'
        print self
        print x+y
        time.sleep(3)
        return x+y





# coding:utf8

"""第三方模块接入点"""

import tornado
from tornado.web import UIModule
from tornado import web, iostream
from tornado.ioloop import IOLoop
from tornado.locks import Condition
from tornado.options import options
from tornado.process import Subprocess
from tornado.concurrent import run_on_executor, TracebackFuture
from tornado import gen, web, httpserver, ioloop
from tornado.gen import coroutine, Return, Task, sleep
from tornado.httpclient import HTTPClient, AsyncHTTPClient, HTTPError, HTTPRequest

from kombu import Queue, Exchange
from celery.exceptions import Ignore
from celery import Task as CeleryTask, platforms, Celery, task, states

from thrift.server import TServer
from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TCompactProtocol

from subprocess32 import Popen

from docopt import docopt

import pymysql
import tornado_mysql

from zeep import CachingClient
from zeep.exceptions import Error
from zeep.wsse.username import UsernameToken
from zeep.transports import Transport


# coding:utf8

import unittest


from fastweb import ioloop
from fastweb.loader import app
from fastweb.web import AsyncHTTPClient, Request, coroutine


app.load_recorder('log/app.log', system_log_path='log/sys.log', system_level='DEBUG')


class AsynComponentTest(unittest.TestCase):

    @coroutine
    def do(self):

        request = Request(method='GET', url='http://www.baidu.com')
        response = yield AsyncHTTPClient().fetch(request)
        print response.body

    def runTest(self):
        ioloop.IOLoop.instance().run_sync(self.do)


if __name__ == '__main__':
    unittest.main()



# coding:utf8

"""系统全局加载模块
外部调用想要影响fastweb行为,必须通过改模块中的方法
所有工作都是在启动前完成,外部导入全部使用全路径引用,防止错误的引入
"""

import json

from accesspoint import ioloop

import fastweb.manager
from fastweb.util.tool import timing
from fastweb.util.configuration import Configuration
from fastweb.util.log import setup_logging, getLogger, recorder, check_logging_level, set_record_color


__all__ = ['app', 'SyncPattern', 'AsynPattern']
DEFAULT_APP_LOG_PATH = 'fastweb@application.log'
DEFAULT_SYS_LOG_PATH = 'fastweb@system.log'


class Loader(object):
    """系统全局加载器
    """

    def __init__(self):
        # 配置系统
        self.configs = None
        self.configer = None

        # 日志系统
        self.system_recorder = None
        self.application_recorder = None

        # 系统错误码
        self.errcode = None

        # 日志是否被设置过
        self.bRecorder = False

    def load_recorder(self, application_log_path=DEFAULT_APP_LOG_PATH, system_log_path=DEFAULT_SYS_LOG_PATH, logging_setting=None,
                      application_level='DEBUG', system_level='DEBUG', logging_colormap=None):
        """加载日志对象

        需要最先加载,因为其他加载都需要使用recorder
        其他server启动时会默认加载一遍，用户没有特殊需求可以不加载

        :parameter:
          - `application_log_path`: 应用日志路径
          - `system_log_path`: 系统日志路径,默认系统日志路径和应用日志路径相同
          - `logging_setting_path`: 默认从fastweb.settting.default_logging.yaml获取配置,可以指定为自定义的日志配置,必须有application_recorder和system_recorder
          - `logging_setting`: 自定以logging配置
          - `application_level`: 应用日志输出级别
          - `system_level`: 系统日志输出级别
          - `logging_colormap`: 输出日志颜色
        """

        if not logging_setting:
            from fastweb.setting.default_logging import DEFAULT_LOGGING_SETTING
            logging_setting = DEFAULT_LOGGING_SETTING

        logging_setting['handlers']['application_file_time_handler']['filename'] = application_log_path
        logging_setting['handlers']['system_file_size_handler']['filename'] = system_log_path

        if application_level:
            check_logging_level(application_level)
            logging_setting['loggers']['application_recorder']['level'] = application_level

        if system_level:
            check_logging_level(system_level)
            logging_setting['loggers']['system_recorder']['level'] = system_level

        setup_logging(logging_setting)

        self.system_recorder = getLogger('system_recorder')
        self.application_recorder = getLogger('application_recorder')

        if logging_colormap:
            set_record_color(logging_colormap)

        self.bRecorder = True
        recorder('INFO',
                 'load recorder configuration\n{conf}\n\napplication log: {app_path} [{app_level}]\nsystem log: {sys_path} [{sys_level}]'.format(
                  conf=json.dumps(logging_setting, indent=4), app_path=application_log_path,
                  app_level=application_level, sys_path=system_log_path, sys_level=system_level))

    def load_configuration(self, backend='ini', **setting):
        """加载配置文件

        :parameter:
          - `backend`: 配置方式,目前支持ini
          - `setting`: 该格式需要的设置参数
        """

        self.configer = Configuration(backend, **setting)
        self.configs = self.configer.configs

        recorder('INFO', 'load configuration\nbackend:\t{backend}\nsetting:\t{setting}\nconfiguration:\t{config}'.format(backend=backend,
                                                                                                                         setting=setting,
                                                                                                                         config=self.configs))

    @staticmethod
    def load_component(layout, backend='ini', **setting):
        """加载组件管理器

        可以进行多次加载

        :parameter:
          - `layout`: 当前调用的层次，web, service, task
          - `backend`: 配置方式,目前支持ini
          - `setting`: 该格式需要的设置参数
        """

        layout = layout.lower()
        configer = Configuration(backend, **setting)

        # 加载需要管理连接池的组件
        recorder('INFO', 'load connection component start')
        with timing('ms', 10) as t:
            if layout in ['service', 'task']:
                fastweb.manager.SyncConnManager.setup(configer)
            elif layout in ['web']:
                fastweb.manager.AsynConnManager.configer = configer
                ioloop.IOLoop.current().run_sync(fastweb.manager.AsynConnManager.setup)
        recorder('INFO', 'load connection component successful -- {time}'.format(time=t))

        # 加载不需要管理连接池的组件
        recorder('INFO', 'load component start')
        with timing('ms', 10) as t:
            fastweb.manager.Manager.setup(layout, configer)
        recorder('INFO', 'load component successful -- {time}'.format(time=t))
        return configer

    def load_errcode(self, errcode=None):
        """加载系统错误码

        :parameter:
          - `errcode`:自定义错误码
        """

        if errcode:
            self.errcode = errcode
        else:
            from fastweb.setting.default_errcode import ERRCODE
            self.errcode = ERRCODE

        recorder('INFO', 'load errcode\n{errcode}'.format(errcode=json.dumps(self.errcode, indent=4)))
        return self.errcode


app = Loader()



# coding:utf8

from fastweb.accesspoint import ioloop, coroutine
from fastweb.component.task import AsynTask, SyncTask

setting = {'_name': 'task test',
           'broker': 'amqp://guest:guest@localhost:5672//',
           'backend': 'redis://localhost/0',
           'task_class': 'some_tasks.add.Add',
           'queue': 'test_task_queue',
           'exchange': 'test_task_exchange',
           'routing_key': 'test_task_routing_key'}


class TestAsynTask(object):
    def test_call(self):
        task = AsynTask(setting)

        @coroutine
        def _call():
            r = yield task.call((1, 2))
            assert r == 3
        ioloop.IOLoop.current().run_sync(_call)

    def test_call_async(self):
        task = AsynTask(setting)

        @coroutine
        def _call():
            r = yield task.call_async((1, 2))
            print(r)
        ioloop.IOLoop.current().run_sync(_call)


class TestSyncTask(object):
    def test_call(self):
        task = SyncTask(setting)
        assert task.call((1, 2)) == 3

    def test_call_async(self):
        task = SyncTask(setting)
        print(task.call_async((1, 2)))




# coding:utf8

from fastweb.task import Worker
from fastweb.service import Service
from fastweb.component.task import AsynTask, SyncTask

# 不区分同步异步的组件
COMPONENTS = [('worker', Worker),
              ('service', Service)]
# 同步组件
SYNC_COMPONENTS = [('task', SyncTask)]
# 异步组件
ASYN_COMPONENTS = [('task', AsynTask)]



# coding:utf8


"""测试subprocess性能"""

import shlex
import subprocess

from gevent.pool import Pool
import gevent.subprocess

from fastweb.util.tool import timing
from fastweb.util.thread import FThread


def call_subprocess(obj):
    returncode = subprocess.call(shlex.split('ls'))
    print 'returncode: {}'.format(returncode)


def call_subprocess_gevent():
    returncode = gevent.subprocess.call(shlex.split('ls'))
    print returncode


if __name__ == '__main__':

    b = True

    x = 1000

    with timing('s', 8) as t:
        if b:
            for i in range(x):
                FThread("Thread-{}".format(i), call_subprocess, frequency=1).start()
            FThread.stop()
        else:
            pool = Pool(x)
            for i in range(x):
                pool.spawn(call_subprocess_gevent)
            pool.join()

    print 'total cost: {}'.format(t)





# coding:utf8


from fastweb import app


class TestLoader(object):

    def test_load_recorder(self):
        app.load_recorder()
        app.load_recorder('app.log')
        app.load_recorder('app.log', system_log_path='sys.log')
        app.load_recorder('app.log', system_log_path='sys.log')
        app.load_recorder('app.log', system_log_path='sys.log', application_level='INFO')
        app.load_recorder('app.log', system_log_path='sys.log', system_level='INFO')

    def test_load_configuration(self):
        app.load_configuration(backend='ini', path='fastweb/test/config/config.ini')
        assert app.configs['default_config']['name'] == 'xxxxx'

    def test_load_component(self):
        app.load_component(layout='web', backend='ini', path='fastweb/test/config/component.ini')

    def test_load_errcode(self):
        app.load_errcode()




# coding:utf8

"""连接池模块"""

import json
import copy
from Queue import Empty, Full
from multiprocessing import Queue

from accesspoint import coroutine, ioloop, Return

from fastweb.util.log import recorder
from fastweb.exception import PoolError
from fastweb.util.thread import FThread

DEFAULT_TIMEOUT = 50000
DEFAULT_MAXCONN = 100


class ConnectionPool(object):
    """连接池"""

    def __init__(self, cls, setting, size, name, awake=DEFAULT_TIMEOUT, maxconnections=DEFAULT_MAXCONN):
        """设置连接池

        连接池创建时尽量早的报错,运行过程中尽量去修复错误

        :parameter:
          - `cls`:连接池中实例化的类
          -`setting`:参数
          -`size`:连接池大小
          -`name`:连接池名字
          -`timeout`:连接最大超时时间,尝试重连时间
        """

        self._cls = cls
        self._name = name
        self._size = int(size)
        self._timeout = int(awake) if awake else DEFAULT_TIMEOUT
        self._setting = setting
        self._maxconnections = int(maxconnections) if maxconnections else DEFAULT_MAXCONN
        self._pool = Queue(self._maxconnections)
        self._rescue_thread = None

    def remove_connection(self, connection):
        """移除连接"""
        pass

    def lend_connection(self, timeout):
        """租用连接

        :parameter:
          `timeout`:租用时间,超过租用时间自动归还
        """
        pass

    def return_connection(self, connection):
        """归还连接

        :parameter:
          - `connection`:连接"""

        self._pool.put_nowait(connection)
        recorder('DEBUG',
                 '<{name}> return connection {conn}, total connections {count}'.format(name=self._name,
                                                                                       conn=connection,
                                                                                       count=self._pool.qsize()))

    def rescue(self):
        """独立线程进行连接恢复"""

        raise NotImplementedError


class SyncConnectionPool(ConnectionPool):
    def __str__(self):
        return '<{name}|SyncConnectionPool>'.format(name=self._name)

    def _create_connection(self):
        """创建连接"""

        return self._cls(self._setting).set_name(self._name).connect()

    def add_connection(self):
        """同步增加连接"""

        connection = self._create_connection()
        try:
            self._pool.put_nowait(connection)
        except Full:
            recorder('ERROR', '<{name}> connection pool is full'.format(name=self._name))
            raise PoolError

        return connection

    def create(self):
        """同步创建连接池"""

        recorder('DEBUG', 'synchronize connection pool create start <{name}>\n{setting}'.format(name=self._name,
                                                                                                setting=json.dumps(
                                                                                                    self._setting,
                                                                                                    indent=4)))
        for _ in range(self._size):
            self.add_connection()
        self.rescue()
        recorder('DEBUG', 'synchronize connection pool create successful <{name}>'.format(name=self._name))

    def rescue(self):
        self._rescue_thread = FThread(name='rescue', task=self._rescue, period=self._timeout, frequency=-1)
        self._rescue_thread.start()

    def _rescue(self, thread):
        """同步恢复连接
        目前先全量恢复
        """

        recorder('INFO', '{thread} <{name}> rescue connection start'.format(thread=thread, name=self._name))
        unused_pool = copy.deepcopy(self._pool)
        for conn in iter(unused_pool, None):
            if conn:
                conn.ping()
            else:
                break
        recorder('INFO', '{thread} <{name}> rescue connection successful'.format(thread=thread, name=self._name))

    def get_connection(self):
        """获取连接"""

        try:
            connection = self._pool.get_nowait()
        except Empty:
            connection = self.add_connection()
            recorder('WARN', '<{name}> connection pool is empty,create a new connection {conn}'.format(name=self._name,
                                                                                                       conn=connection))
            return self.get_connection()

        recorder('DEBUG', '{obj} get connection {conn} {id}, left connections {count}'.format(obj=self, conn=connection,
                                                                                              id=id(connection),
                                                                                              count=self._pool.qsize()))
        return connection


class AsynConnectionPool(ConnectionPool):
    def __str__(self):
        return '<{name}|AsynConnectionPool>'.format(name=self._name)

    @coroutine
    def _create_connection(self):
        """创建连接"""

        connection = yield self._cls(self._setting).set_name(self._name).connect()
        raise Return(connection)

    @coroutine
    def create(self):
        """异步创建连接池"""

        recorder('DEBUG', 'asynchronous connection pool create start <{name}>\n{setting}'.format(name=self._name,
                                                                                                 setting=json.dumps(
                                                                                                     self._setting,
                                                                                                     indent=4)))
        for _ in range(self._size):
            yield self.add_connection()
        self.rescue()
        recorder('DEBUG', 'asynchronous connection pool create successful <{name}>'.format(name=self._name))

    @coroutine
    def add_connection(self):
        """同步增加连接"""

        connection = yield self._create_connection()
        try:
            self._pool.put_nowait(connection)
        except Full:
            recorder('ERROR', '<{name}> connection pool is full'.format(name=self._name))
            raise PoolError

    def rescue(self):
        """异步恢复连接
        目前全量恢复
        """

        def on_reconnect(future):
            if future.exception:
                print future.exc_info

        def on_rescue():
            recorder('INFO', '<{name}> rescue connection start'.format(name=self._name))
            unused_pool = copy.deepcopy(self._pool)
            for conn in iter(unused_pool, None):
                if conn:
                    future = conn.ping()
                    ioloop.IOLoop.current().add_future(future, on_reconnect)
                else:
                    break
            recorder('INFO', '<{name}> rescue connection successful'.format(name=self._name))
            self.rescue()

        ioloop.IOLoop.current().add_timeout(ioloop.IOLoop.current().time() + self._timeout, on_rescue)

    def _scale(self, thread):
        recorder('WARN', '{thread} {obj} scale connection pool start'.format(thread=thread, obj=self))
        scale_loop = ioloop.IOLoop(make_current=True)
        scale_loop.run_sync(self.create)
        recorder('WARN', '{thread} {obj} scale connection pool successful'.format(thread=thread, obj=self))
        scale_loop.start()

    def scale_connections(self):
        scale_thread = FThread(name='scale', task=self._scale, frequency=-1)
        scale_thread.start()
        scale_thread.join()

    def get_connection(self):
        """获取连接"""

        # TODO:连接池扩展机制问题
        try:
            connection = self._pool.get(block=True)
            if self._pool.qsize() < 2:
                self.scale_connections()
        except Empty:
            recorder('CRITICAL',
                     '<{name}> connection pool is empty,please use service to separate your database operation'.format(
                         name=self._name))
            raise PoolError

        recorder('DEBUG', '{obj} get connection {conn} {id}, left connections {count}'.format(obj=self, conn=connection,
                                                                                              id=id(connection),
                                                                                              count=self._pool.qsize()))
        return connection



# coding:utf8


import unittest

import init
from fastweb.util.tool import Retry, RetryPolicy


class HelloError(Exception):
    pass


class RetryTest(unittest.TestCase):

    def runTest(self):

        policy = RetryPolicy(3, HelloError, interval=3, delay=2)

        policy1 = RetryPolicy(4, HelloError)

        def hello(time):
            print time
            raise policy

        def hello1(time):
            print time
            raise policy1

        retry = Retry('hello', hello, 1)
        retry.run()

        retry1 = Retry('hello1', hello1, 1)
        retry1.run()



#
# Autogenerated by Thrift Compiler (0.10.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:tornado
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
import sys
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
from tornado import gen
from tornado import concurrent


class Iface(object):
    def sayHello(self):
        pass


class Client(Iface):
    def __init__(self, transport, iprot_factory, oprot_factory=None):
        self._transport = transport
        self._iprot_factory = iprot_factory
        self._oprot_factory = (oprot_factory if oprot_factory is not None
                               else iprot_factory)
        self._seqid = 0
        self._reqs = {}
        self._transport.io_loop.spawn_callback(self._start_receiving)

    @gen.engine
    def _start_receiving(self):
        while True:
            try:
                frame = yield self._transport.readFrame()
            except TTransport.TTransportException as e:
                for future in self._reqs.values():
                    future.set_exception(e)
                self._reqs = {}
                return
            tr = TTransport.TMemoryBuffer(frame)
            iprot = self._iprot_factory.getProtocol(tr)
            (fname, mtype, rseqid) = iprot.readMessageBegin()
            method = getattr(self, 'recv_' + fname)
            future = self._reqs.pop(rseqid, None)
            if not future:
                # future has already been discarded
                continue
            try:
                result = method(iprot, mtype, rseqid)
            except Exception as e:
                future.set_exception(e)
            else:
                future.set_result(result)

    def sayHello(self):
        self._seqid += 1
        future = self._reqs[self._seqid] = concurrent.Future()
        self.send_sayHello()
        return future

    def send_sayHello(self):
        oprot = self._oprot_factory.getProtocol(self._transport)
        oprot.writeMessageBegin('sayHello', TMessageType.CALL, self._seqid)
        args = sayHello_args()
        args.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def recv_sayHello(self, iprot, mtype, rseqid):
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = sayHello_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "sayHello failed: unknown result")


class Processor(Iface, TProcessor):
    def __init__(self, handler):
        self._handler = handler
        self._processMap = {}
        self._processMap["sayHello"] = Processor.process_sayHello

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            return self._processMap[name](self, seqid, iprot, oprot)

    @gen.coroutine
    def process_sayHello(self, seqid, iprot, oprot):
        args = sayHello_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = sayHello_result()
        result.success = yield gen.maybe_future(self._handler.sayHello())
        oprot.writeMessageBegin("sayHello", TMessageType.REPLY, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class sayHello_args(object):

    thrift_spec = (
    )

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, (self.__class__, self.thrift_spec))
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, (self.__class__, self.thrift_spec)))
            return
        oprot.writeStructBegin('sayHello_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class sayHello_result(object):
    """
    Attributes:
     - success
    """

    thrift_spec = (
        (0, TType.STRING, 'success', 'UTF8', None, ),  # 0
    )

    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, (self.__class__, self.thrift_spec))
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRING:
                    self.success = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, (self.__class__, self.thrift_spec)))
            return
        oprot.writeStructBegin('sayHello_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRING, 0)
            oprot.writeString(self.success.encode('utf-8') if sys.version_info[0] == 2 else self.success)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)



import code
import readline
import atexit
import os


class HistoryConsole(code.InteractiveConsole):
    def __init__(self, locals=None, filename="<console>",
                 histfile=os.path.expanduser("~/.console-history")):
        code.InteractiveConsole.__init__(self, locals, filename)
        self.init_history(histfile)

    def init_history(self, histfile):
        readline.parse_and_bind("tab: complete")
        if hasattr(readline, "read_history_file"):
            try:
                readline.read_history_file(histfile)
            except IOError:
                pass
            atexit.register(self.save_history, histfile)

    def save_history(self, histfile):
        readline.set_history_length(1000)
        readline.write_history_file(histfile)


if __name__ == '__main__':
    while True:
        c = HistoryConsole().raw_input()
        print c



# coding:utf8

"""服务层模块"""

import inspect

from accesspoint import TServer, TSocket, TTransport, TCompactProtocol

from fastweb import app
from fastweb.manager import Manager
from fastweb.util.tool import timing
from fastweb.util.log import recorder
from fastweb.component import Component
from fastweb.util.process import FProcess
from fastweb.components import SyncComponents
from fastweb.util.python import load_module, to_iter, load_object


__all__ = ['start_service_server', 'ABLogic']
DEFAULT_THREADPOOL_SIZE = 1000


class Service(Component):
    """服务类类

    Thrift服务端实现

    关于Thrift

    TftRpc是Thrift客户端的实现，fastweb.service.Service是Thrift服务端的实现

    关于Transport:
        目前我们选用的TFrameTransport，主要是为了兼容异步的连接，Thrift的异步链接必须使用TFrameTransport

    关于Protocal:
        目前选择的是TCompactProtocol，一种压缩的高性能二进制编码，这种传输方式类似于ProtocolBuffer，性能会比TBinaryProtocol高

    关于Server：
        目前选择的是TThreadPoolServer，多线程的模式处理高并发时效率很高，但是会伴随着系统性能的大量开销。
        TNonBlockingServer为非阻塞IO的多线程模式，使用少量线程既可以完成大并发量的请求响应，必须使用TFramedTransport。
        TNonblockingServer能够使用少量线程处理大量并发连接，但是延迟较高；TThreadedServer的延迟较低。
        实际中，TThreadedServer的吞吐量可能会比TNonblockingServer高，但是TThreadedServer的CPU占用要比TNonblockingServer高很多。

    :parameter:
      - `port`: 启动端口号,如果为列表则生成多个进程,配置必填项
      - `thrift_module`:thrift生成模块路径，系统可查找或相对启动路径类路径,只能有一个
      - `handlers`: 处理具体业务的handler类,可以为列表或单个类
      - `daemon`: 是否以守护进程的形式启动
      - `active`: 是否可用
      - `size`: 线程池大小
    """

    eattr = {'port': int, 'thrift_module': str, 'handlers': str}
    oattr = {'size': int, 'daemon': bool, 'active': bool}

    def __init__(self, setting):
        super(Service, self).__init__(setting)

        # 设置service属性
        self._port = self.setting['port']
        self._thrift_module = self.setting['thrift_module']
        self._handlers = self.setting['handlers']
        handlers = to_iter(self._handlers)
        self._daemon = self.setting.get('daemon', True)
        self._active = self.setting.get('active', True)
        self.size = self.setting.get('size', DEFAULT_THREADPOOL_SIZE)

        # 合并多个handler为一个,并自动集成ABLogic
        handlers = tuple(load_object(handler) for handler in handlers)

        try:
            self._handlers = type('Handler', handlers + (ABLogic, ), {})
        except TypeError as e:
            self.recorder('CRITICAL', 'handler conflict ({e})'.format(e=e))

    def __str__(self):
        return '<Service|{name} {port} {module}->{handler}>'.format(name=self.name,
                                                                    port=self._port,
                                                                    module=self._thrift_module,
                                                                    handler=self._handlers)

    def start(self):
        """微服务开始

        生成一个微服务

        :parameter:
         - `handler`:AbLogic列表
        """

        def process_proxy(processor):
            for _func_name, _func in processor._processMap.items():
                def anonymous(p, seq, ipo, opo):
                    oproc = getattr(module, 'Processor')(handler=self._handlers())
                    oproc._handler.requestid = seq if len(str(seq)) > 8 else oproc._handler.requestid
                    oproc._handler.recorder('IMPORTANT', '{obj}\nremote call [{name}]'.format(obj=self, name=_func_name))
                    with timing('ms', 8) as t:
                        _func(oproc, seq, ipo, opo)
                    oproc._handler.release()
                    oproc._handler.recorder('IMPORTANT', '{obj}\nremote call [{name}] success -- {t}'.format(obj=self,
                                                                                                             name=_func_name,
                                                                                                             t=t))
                processor._processMap[_func_name] = anonymous

        # 将所有的handlers合并成一个handler
        module = load_module(self._thrift_module)
        processor = getattr(module, 'Processor')(handler=None)
        process_proxy(processor)
        transport = TSocket.TServerSocket(port=self._port)
        tfactory = TTransport.TFramedTransportFactory()
        pfactory = TCompactProtocol.TCompactProtocolFactory()
        server = TServer.TThreadPoolServer(processor, transport, tfactory, pfactory, daemon=self._daemon)
        server.setNumThreads(self.size)

        try:
            if self._active:
                recorder('INFO', '{svr} start at <{port}> threadpool size <{size}>'.format(svr=self, port=self._port,
                                                                                           size=self.size))
                server.serve()
        except KeyboardInterrupt:
            recorder('INFO', '{svr} stop at <{port}>'.format(svr=self, port=self._port))


class ABLogic(SyncComponents):
    """基础逻辑类"""

    def __init__(self):
        super(ABLogic, self).__init__()


def start_service_server():
    """强制使用config的方式来配置微服务

    port: 启动端口号,如果为列表则生成多个进程,配置必填项
    thrift_module:  thrift生成的模块类路径,系统可查找或相对启动路径类路径,只能有一个
    handlers: 处理具体业务的handler类,可以为列表或单个类
    daemon: 是否以守护进程的形式启动
    active: 是否可用
    size: 线程池大小

    :parameter:
      - `config_path`:配置文件路径
    """

    if not app.bRecorder:
        app.load_recorder()

    # 将调用者路径加入到包查找路径中
    import sys
    sys.path.append(inspect.stack()[1][1])
    del sys

    services = Manager.get_classified_components('service')

    if len(services) == 1:
        # 只有一个service时，只启动一个进程
        services[0].start()
    else:
        # 有多个service会启动多进程
        # 多进程模式下不能使用pdb.set_trace的方式调试
        for service in services:
            process = FProcess(name='ServiceProcess', task=service.start)
            process.start()



# coding:utf8

"""异常模块"""


class FastwebException(Exception):
    """fastweb框架异常"""
    pass


class ComponentError(FastwebException):
    """组件错误"""
    pass


class HttpError(ComponentError):
    """HTTP访问错误"""
    pass


class SubProcessError(ComponentError):
    """子进程调用错误"""
    pass


class SubProcessTimeoutError(ComponentError):
    """子进程调用超时错误"""
    pass


class MysqlError(ComponentError):
    """mysql组件错误"""
    pass


class RedisError(ComponentError):
    """redis组件错误"""
    pass


class MongoError(ComponentError):
    """mongo组件错误"""
    pass


class RpcError(ComponentError):
    """rpc组件错误"""
    pass


class ConfigurationError(FastwebException):
    """配置错误"""
    pass


class ConnectionPoolError(FastwebException):
    """连接池错误"""
    pass


class ManagerError(FastwebException):
    """管理器错误"""
    pass


class ParameterError(FastwebException):
    """参数错误"""
    pass


class ServiceError(FastwebException):
    """服务层错误"""
    pass


class TaskError(ComponentError):
    """任务错误"""
    pass


class ThriftParserError(ComponentError):
    """thrift文件解析错误"""
    pass


class SoapError(ComponentError):
    """Soap 错误"""
    pass


class PoolError(ComponentError):
    """连接池错误"""
    pass




# coding:utf8


import re
import collections
import ConfigParser

from fastweb.util.log import recorder
from fastweb.util.python import to_iter
from fastweb.exception import ParameterError


class Configuration(object):
    """配置解析

    TODO:支持多种配置后端,后续改造
    """

    parser = {'ini': '_ini_parser', 'confd': '_confd_parser'}

    def __init__(self, backend, **setting):
        """获取配置信息

        configs:{'section': {setting}}

        :parameter:
          - `backend`:后端类型
          - `setting`:根据后端类型相关的配置项
        """

        self.configs = None
        self.get_configs_from_backend(backend, setting)

    @staticmethod
    def _check_setting(eattr, setting):
        """检查配置

        :parameter:
          - `eattr`:必要配置
          - `setting`:传入参数,参数不足抛出ParameterError
        """

        for attr in eattr:
            v = setting.get(attr)
            if not v:
                recorder('CRITICAL', 'configuration backend setting error! right options {options}'.format(options=eattr))
                raise ParameterError

    def _ini_parser(self, setting):
        """ini配置文件解析

        :parameter:
          - `setting`:传入参数
        """

        eattr = ['path']
        self._check_setting(eattr, setting)
        path = setting.get('path')
        cf = ConfigParser.ConfigParser()
        cf.read(path)
        configs = collections.defaultdict(dict)

        for section in cf.sections():
            options = cf.items(section)
            for key, value in options:
                value = self._type_conversion(value)
                configs[section][key] = value
        return configs

    def _confd_parser(self, setting):
        """confd"""
        pass

    @staticmethod
    def _type_conversion(v):
        """尝试转换配置文件中的value类型"""

        # 数字解析
        if v.isdigit():
            return int(v)

        # 列表解析
        l = v.split(',')
        if len(l) > 1:
            return [i for i in l if i]

        # boolean解析
        lv = v.lower()
        if lv == 'yes':
            return True
        elif lv == 'no':
            return False
        else:
            return v

    def get_configs_from_backend(self, backend, setting):
        """从不同的backend获取相同格式的配置

        :parameter:
          - `backend`:后端类型,后端类型不正确抛出ParameterError
          - `setting`:传入参数
        """

        parser = self.parser.get(backend)
        if parser:
            self.configs = getattr(self, parser)(setting)
        else:
            raise ParameterError

    def get_components(self, components):
        """根据组件名称获取组件配置

        :parameter:
          - `components`:组件名称

        :return:
          {section: {'component': component_name, 'object': component_type}}
        """

        match_components = {}
        components = to_iter(components)

        for component in components:
            component_exp = r'(%s):(\w*)' % component
            exp = re.compile(component_exp)

            for section in self.configs.keys():
                match = exp.match(section)
                if match:
                    match_components[section] = {'component': match.group(1), 'object': match.group(2)}

        return match_components



# coding:utf8

"""FastWeb is a Python fast-building web frame refered by Tornado,Celery,Thrift"""

# :copyright: (c) 2016-2017 Bslience.  All rights reserved.

from __future__ import absolute_import, division, print_function, with_statement

from fastweb.loader import app
from fastweb.accesspoint import options


__all__ = ['options', 'app']






# coding:utf8

import os
import six

from thrift import TTornado
from thrift.protocol import TCompactProtocol
from thrift.transport import TTransport, TSocket
from fastweb.accesspoint import coroutine, Return

from fastweb.util.log import recorder
from fastweb.component import Component
from fastweb.exception import RpcError, ConfigurationError
from fastweb.util.python import AsynProxyCall, ExceptionProcessor, load_module


"""
关于Thrift

TftRpc是Thrift客户端的实现，fastweb.service.Service是Thrift服务端的实现

关于Transport:
    Thrift官方实现了多种Transport，目前我们选用的TFrameTransport，主要是为了兼容异步的连接，Thrift的异步链接必须使用TFrameTransport
    
关于Protocal:
    Thrift官方实现了多种Protocal，目前选择的是TCompactProtocol，一种压缩的高性能二进制编码，这种传输方式类似于ProtocolBuffer，性能会
    比TBinaryProtocol高
"""


class TftRpc(Component):

    eattr = {'host': str, 'port': int, 'thrift_module': str}

    def __init__(self, setting):
        super(TftRpc, self).__init__(setting)

        self._client = None
        self._transport = None
        self.other = None

        # 将用户启动的路径加入sys.path，用户普遍理解路径都是从当前目录进行理解
        import sys
        sys.path.append(os.getcwd())
        del sys

    def connect(self):
        raise NotImplementedError

    def reconnect(self):
        raise NotImplementedError

    def ping(self):
        pass

    def _recorder(self, level, msg):
        return self.owner.recorder(level, msg) if self.owner else recorder(level, msg)


class SyncTftRpc(TftRpc):
    """Thrift Rpc同步组件"""

    def __str__(self):
        return '<SyncThriftRpc {name} {host} {port} {module}>'.format(
            host=self.host,
            port=self.port,
            module=self.thrift_module,
            name=self.name)

    def connect(self):
        if isinstance(self.thrift_module, six.string_types):
            module = load_module(self.thrift_module)
        else:
            self.recorder('ERROR', '{obj} module [{module}] load error'.format(obj=self,
                                                                               module=self.thrift_module))
            raise ConfigurationError

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self._transport = TSocket.TSocket(self.host, self.port)
            self._transport = TTransport.TFramedTransport(self._transport)
            protocol = TCompactProtocol.TCompactProtocol(self._transport)
            self._client = getattr(module, 'Client')(protocol)
            self._transport.open()
            self.recorder('INFO', '{obj} connect successful'.format(obj=self))
        except TTransport.TTransportException as e:
            self.recorder('ERROR', '{obj} connect error ({e})'.format(obj=self, e=e)) if self.recorder else recorder('ERROR', '{obj} connect error ({e})'.format(obj=self, e=ex))
            raise RpcError

        return self

    def reconnect(self):
        pass

    def __getattr__(self, name):
        self._client._seqid = int(self.owner.requestid) if self.owner else 0
        if hasattr(self._client, name):
            self._recorder('INFO', 'call {obj} {name} start'.format(obj=self, name=name))
            r = getattr(self._client, name)
            self._recorder('INFO', 'call {obj} {name} success'.format(obj=self, name=name))
            return r
        else:
            raise AttributeError

    def close(self):
        self._transport.close()


class AsynTftRpc(TftRpc):
    """Thrift Rpc异步组件"""

    def __str__(self):
        return '<AsynTftRpc {host} {port} {module} {name}>'.format(
            host=self.host,
            port=self.port,
            module=self.thrift_module,
            name=self.name)

    @coroutine
    def connect(self):
        """建立连接"""

        if isinstance(self.thrift_module, six.string_types):
            module = load_module(self.thrift_module)

        self.recorder('INFO', '{obj} connect start'.format(obj=self))
        self._transport = TTornado.TTornadoStreamTransport(self.host, self.port)
        yield self._connect()
        self.recorder('INFO', '{obj} connect successful'.format(obj=self))
        protocol = TCompactProtocol.TCompactProtocolFactory()
        self._client = getattr(module, 'Client')(self._transport, protocol)
        self.other = self._client
        raise Return(self)

    @coroutine
    def _connect(self):
        try:
            yield self._transport.open()
        except TTransport.TTransportException as e:
            self.recorder('ERROR', '{obj} connect error ({e})'.format(obj=self, e=e)) if self.recorder else recorder('ERROR', '{obj} connect error ({e})'.format(obj=self, e=ex))
            raise RpcError

    def reconnect(self):
        pass

    def __getattr__(self, name):
        """获取远程调用方法"""
        self._client._seqid = int(self.owner.requestid) if self.owner else 0
        exception_processor = ExceptionProcessor(AttributeError, self._connect)

        if hasattr(self._client, name):
            self._recorder('INFO', 'call {obj} {name} start'.format(obj=self, name=name))
            r = AsynProxyCall(self, name, throw_exception=RpcError, exception_processor=exception_processor)
            self._recorder('INFO', 'call {obj} {name} success'.format(obj=self, name=name))
            return r
        else:
            raise AttributeError

    def close(self):
        """关闭连接"""
        self._transport.close()



# coding:utf8

import redis
import shlex

import tornadis
from tornadis.exceptions import ConnectionError as torConnectionError
from redis.exceptions import ConnectionError, TimeoutError, ResponseError

from fastweb.accesspoint import coroutine, Return

import fastweb.util.tool as tool
from fastweb.exception import RedisError
from fastweb.component import Component


DEFAULT_DB = 0
DEFAULT_PORT = 6379
DEFAULT_TIMEOUT = 5
DEFAULT_CHARSET = 'utf8'


class Redis(Component):
    """Redis基类"""

    eattr = {'host': str}
    oattr = {'port': int, 'password': str, 'db': int, 'timeout': int, 'charset': str}

    def __init__(self, setting):
        self.db = DEFAULT_DB
        self.port = DEFAULT_PORT
        self.timeout = DEFAULT_TIMEOUT
        self.charset = DEFAULT_CHARSET

        super(Redis, self).__init__(setting)

        self._client = None

    @staticmethod
    def _parse_response(response):
        """解析response"""

        if response == 'OK':
            return True
        elif isinstance(response, tornadis.ClientError):
            raise RedisError(response)
        else:
            return response

    def reconnect(self):
        pass

    def ping(self):
        pass


class SyncRedis(Redis):
    """同步redis
       线程不安全"""

    def __reduce__(self):
        return SyncRedis, (self.setting,)

    def __str__(self):
        return '<SyncRedis {name} {host} {port} {db} {charset}>'.format(
            name=self.name, host=self.host, port=self.port, db=self.db, charset=self.charset)

    def connect(self):
        self.setting['socket_timeout'] = self.setting.pop('timeout', None)

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self._client = redis.StrictRedis(**self.setting)
            self.recorder('INFO', '{obj} connect successful'.format(obj=self))
        except ConnectionError as e:
            self.recorder('ERROR', '{obj} connect failed [{msg}]'.format(obj=self, msg=e))
            raise RedisError
        return self

    def query(self, command):
        """命令行操作
           ConnectionError可能是超出连接最大数
           TimeoutError可能是连接不通"""

        try:
            cmd = shlex.split(command)
            self.recorder('INFO', '{obj} query start\n{cmd}'.format(obj=self, cmd=command))
            with tool.timing('s', 10) as t:
                response = self._client.execute_command(*cmd)
            self.recorder('INFO', '{obj} query successful\n{cmd} -- {time}'.format(obj=self, cmd=command, time=t))
        except (ConnectionError, TimeoutError) as e:
            # redis内部对这两种异常进行了重试操作
            self.recorder('ERROR', '{obj} connection error [{msg}]'.format(obj=self, msg=e))
            raise RedisError
        except ResponseError as e:
            self.recorder('ERROR', '{obj} query error [{msg}]'.format(obj=self, msg=e))
            raise RedisError

        return self._parse_response(response)


class AsynRedis(Redis):
    """异步redis组件"""

    def __reduce__(self):
        return AsynRedis, (self.setting,)

    def __str__(self):
        return '<AsynRedis {name} {host} {port} >'.format(
            host=self.host, port=self.port, name=self.name)

    @coroutine
    def connect(self):
        self.setting['connect_timeout'] = self.setting.pop('timeout', None)
        self.setting.pop('charset', None)

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self._client = tornadis.Client(**self.setting)
            future = yield self._client.connect()
            if not future:
                raise RedisError
            self.recorder('INFO', '{obj} connect successful'.format(obj=self))
        except torConnectionError as e:
            self.recorder('ERROR', '{obj} connect failed [{msg}]'.format(obj=self, msg=e))
            raise RedisError
        raise Return(self)

    @coroutine
    def query(self, command):
        """执行redis命令"""

        try:
            cmd = shlex.split(command)
            self.recorder('INFO', '{obj} query start\nCommand: {cmd}'.format(obj=self, cmd=command))
            with tool.timing('s', 10) as t:
                response = yield self._client.call(*cmd)
            self.recorder('INFO', '{obj} query success\nCommand: {cmd}\nResponse: {res} -- {time}'.format(obj=self,
                                                                                                          cmd=command,
                                                                                                          res=response,
                                                                                                          time=t))
        except torConnectionError as e:
            self.recorder('ERROR', '{obj} connection error [{msg}]'.format(obj=self, msg=e))
            raise RedisError

        raise Return(self._parse_response(response))





# coding:utf8

import unittest

from fastweb.loader import app
from fastweb.util.process import FProcess

app.load_recorder('log/app.log', system_log_path='log/sys.log', system_level='DEBUG')


class FProcessTest(unittest.TestCase):

    def task(self):
        import time
        time.sleep(10)

    def runTest(self):
        process = FProcess(name='test', task=self.task)
        process.start()
        print 111111
        process.stop()


if __name__ == '__main__':
    unittest.main()



# coding:utf8


from fastweb.accesspoint import ioloop, coroutine
from fastweb.component.db.rds import SyncRedis, AsynRedis


setting = {'host': 'localhost'}


class TestSyncRedis(object):

    def test_connect(self):
        redis = SyncRedis(setting)
        redis.connect()

    def test_query(self):
        redis = SyncRedis(setting)
        redis.connect()
        assert redis.query('set name jackson')


class TestAsynRedis(object):

    def test_connect(self):
        redis = AsynRedis(setting)
        ioloop.IOLoop.current().run_sync(redis.connect)

    def test_query(self):
        redis = AsynRedis(setting)
        ioloop.IOLoop.current().run_sync(redis.connect)

        @coroutine
        def _query():
            r = yield redis.query('set name jackson')
            assert r
        ioloop.IOLoop.current().run_sync(_query)



# coding:utf8


from fastweb.util.log import recorder
from fastweb.exception import ConfigurationError


# 不可用状态,需要进行准备工作才能使用
UNUSED = 1
# 空闲可用状态
IDLE = 2
# 忙碌不可用状态
USED = 3
# 错误不可用状态
ERROR = 4


class Component(object):
    """组件基类

    组件状态流转:组件实例化后的初始状态为UNUSED,准备工作完成后状态为IDLE，被Manager交出将使用权后状态为USED，收回使用权后状态为IDLE
    无法成为IDLE则状态为ERROR状态

    :parameter:
      - `eattr`:组件必须传递参数以及传递参数类型,如果传递类型与标示类型不同,会尝试进行转换,转换失败抛出ConfigurationError
      - `oattr`:组件可选传递参数以及传递参数类型,如果传递类型与标示类型不同,会尝试进行转换,转换失败抛出ConfigurationError
      - `errprocess`:组件自救方式
    """

    eattr = {}
    oattr = {}
    errprocess = {}

    def __init__(self, setting):
        """构建组件,预检查组件

        :parameter:
          - `setting`:构造组件参数
        """

        self.name = ''
        # 存储原始的参数
        self._setting = setting
        # 存储转换后的参数
        self.setting = {}
        self.owner = None
        self.recorder = recorder
        self.status = UNUSED
        self.exceptions = []

        self._check_attrs()

    def set_name(self, name):
        """设置组件名称

        :parameter:
          - `name`:组件名称
        """

        self.name = name
        return self

    def set_unused(self):
        """设置为不可用状态"""

        self.status = UNUSED
        self.owner = None
        self.recorder = recorder

    def set_idle(self):
        """设置为空闲可用状态"""

        self.status = IDLE
        self.owner = None
        self.recorder = recorder

    def set_used(self, owner):
        """设置为忙碌可用状态

        :parameter:
          - `owner`: 宿主
        """

        self.status = USED
        self.owner = owner
        self.recorder = owner.recorder

    def set_error(self, ex):
        """设置为错误状态,等待回收"""

        self.status = ERROR
        self.owner = None
        self.recorder = recorder
        self.exceptions.append(ex)

    def _check_attrs(self):
        """检查组件属性是否合法"""

        def _convert_attrs(att, t, va):
            """转换参数类型"""

            try:
                va = t(va)
                self.setting[att] = va
            except (ValueError, TypeError) as e:
                recorder('ERROR', "<{attr}> can't convert to {tp} ({e})".format(attr=att, tp=t, name=self.name, e=e))
                raise ConfigurationError

        def _add_attr(att, va):
            """增加成员属性"""

            if hasattr(self, att):
                if va:
                    setattr(self, att, va)
            else:
                setattr(self, att, va)

        for attr, tp in self.eattr.iteritems():
            v = self._setting.get(attr)
            _add_attr(attr, v)
            if v:
                _convert_attrs(attr, tp, v)
            else:
                recorder('ERROR', '<{attr}> is essential attribute of <{obj}>'.format(attr=attr, obj=self.__class__))
                raise ConfigurationError

        for attr, tp in self.oattr.iteritems():
            v = self._setting.get(attr)
            _add_attr(attr, v)
            if v:
                _convert_attrs(attr, tp, v)

    def selfrescue(self):
        """自救"""
        pass



# coding:utf8

import urllib

from fastweb.accesspoint import HTTPRequest
from fastweb.accesspoint import UsernameToken


DEFAULT_TIMEOUT = 5
DEFAULT_RETRY_TIME = 3

__all__ = ['UsernameToken', 'Request', 'SoapRequest']


class Request(HTTPRequest):
    """Http请求对象"""

    def __init__(self, url, method="GET", headers=None, body=None,
                 auth_username=None, auth_password=None, auth_mode=None,
                 connect_timeout=None, request_timeout=None,
                 if_modified_since=None, follow_redirects=None,
                 max_redirects=None, user_agent=None, use_gzip=None,
                 network_interface=None, streaming_callback=None,
                 header_callback=None, prepare_curl_callback=None,
                 proxy_host=None, proxy_port=None, proxy_username=None,
                 proxy_password=None, allow_nonstandard_methods=None,
                 validate_cert=None, ca_certs=None,
                 allow_ipv6=None,
                 client_key=None, client_cert=None, body_producer=None,
                 expect_100_continue=False, decompress_response=None,
                 ssl_options=None, params=None, retry=DEFAULT_RETRY_TIME):
        if params:
            url = '{url}?{params}'.format(url, urllib.urlencode(params))
        if body:
            body = urllib.urlencode(body)
        self.retry = retry
        super(Request, self).__init__(url, method=method, headers=headers, body=body,
                                      auth_username=auth_username, auth_password=auth_password, auth_mode=auth_mode,
                                      connect_timeout=connect_timeout, request_timeout=request_timeout,
                                      if_modified_since=if_modified_since, follow_redirects=follow_redirects,
                                      max_redirects=max_redirects, user_agent=user_agent, use_gzip=use_gzip,
                                      network_interface=network_interface, streaming_callback=streaming_callback,
                                      header_callback=header_callback, prepare_curl_callback=prepare_curl_callback,
                                      proxy_host=proxy_host, proxy_port=proxy_port, proxy_username=proxy_username,
                                      proxy_password=proxy_password, allow_nonstandard_methods=allow_nonstandard_methods,
                                      validate_cert=validate_cert, ca_certs=ca_certs,
                                      allow_ipv6=allow_ipv6,
                                      client_key=client_key, client_cert=client_cert, body_producer=body_producer,
                                      expect_100_continue=expect_100_continue, decompress_response=decompress_response,
                                      ssl_options=ssl_options)

    def __str__(self):
        return '<Request {method} {url} {body}>'.format(method=self.method, url=self.url, body=self.body)


class SoapRequest(object):
    """Soap 请求对象"""

    def __init__(self, wsdl, function, wsse=None, retry=DEFAULT_RETRY_TIME, timeout=DEFAULT_TIMEOUT, **kwargs):
        self.wsdl = wsdl
        self.function = function
        self.wsse = wsse
        self.kwargs = kwargs
        self.retry = retry
        self.timeout = timeout

    def __str__(self):
        return '<SoapRequest {wsdl} {function} {kwargs}>'.format(wsdl=self.wsdl, function=self.function,
                                                                 kwargs=self.kwargs)



# coding:utf8

"""Mysql模块"""


import fastweb.util.tool as tool
from fastweb.component import Component
from fastweb.exception import MysqlError
from fastweb.util.tool import Retry, RetryPolicy
from fastweb.accesspoint import iostream, pymysql, tornado_mysql, coroutine, Return

pymysql.threadsafety = 2
tornado_mysql.threadsafety = 2

DEFAULT_PORT = 3306
DEFAULT_TIMEOUT = 5
DEFAULT_CHARSET = 'utf8'
DEFAULT_AUTOCOMMIT = True


class Mysql(Component):
    """Mysql基类"""

    eattr = {'host': str}
    oattr = {'port': int, 'user': str, 'password': str, 'db': str, 'timeout': int, 'charset': str, 'autocommit': bool}

    def __init__(self, setting):
        self.port = DEFAULT_PORT
        self.charset = DEFAULT_CHARSET
        self.timeout = DEFAULT_TIMEOUT
        self.autocommit = DEFAULT_AUTOCOMMIT

        super(Mysql, self).__init__(setting)

        self._conn = None
        self._cur = None
        # 事务标识
        self._event = False

        # 最后执行sql
        self._sql = None
        # 最后执行sql参数
        self._args = None

        self._prepare()

    def _prepare(self):
        """准备工作"""
        self.setting['passwd'] = self.setting.pop('password', None)
        self.setting['connect_timeout'] = self.setting.pop('timeout', DEFAULT_TIMEOUT)

    def _format_sql(self, sql, args):
        """格式化sql
        占位符不需要严格区分类型，%s是一个好的选择"""

        def type_convert(v):
            if v is None:
                return 'NULL'
            return v

        if isinstance(args, dict):
            self._args = args = {k: type_convert(v) for k, v in args.items()}
            self._sql = sql % args
        elif isinstance(args, tuple):
            self._args = args = tuple([type_convert(v) for v in args])
            self._sql = sql % args
        elif isinstance(args, str):
            self._sql = sql % args
        else:
            self._sql = sql

        return sql

    def connect(self):
        """建立连接"""
        raise NotImplementedError

    def ping(self):
        """维持长连接"""
        raise NotImplementedError

    def reconnect(self):
        """重新建立连接"""
        raise NotImplementedError

    def start_event(self):
        """开始事务"""
        raise NotImplementedError

    def exec_event(self, sql, **kwargs):
        """执行事务"""
        raise NotImplementedError

    def end_event(self):
        """结束事务
        如果不结束,则事务无效果,事务虽然无效但是会占用自增id
        """

        raise NotImplementedError

    def query(self, sql, **kwargs):
        """查询sql"""
        raise NotImplementedError

    def fetch(self):
        """获取一条结果"""
        return self._cur.fetchone()

    def fetchall(self):
        """获取全部结果"""
        return self._cur.fetchall()

    def close(self):
        """关闭连接"""
        raise NotImplementedError

    def rollback(self):
        """事务回滚"""
        raise NotImplementedError

    def commit(self):
        """事务提交"""
        raise NotImplementedError

    @property
    def efficetid(self):
        """返回effectid"""
        return int(self._conn.insert_id())

    @property
    def threadid(self):
        """返回服务端线程id"""
        return int(self._conn.thread_id())


class SyncMysql(Mysql):
    """同步mysql
       线程不安全"""

    def __reduce__(self):
        return SyncMysql, (self.setting,)

    def __str__(self):
        return '<SyncMysql|{name}|{id} {host} {port} {user} {db} {charset}>'.format(
            id=id(self),
            host=self.host,
            port=self.port,
            user=self.user,
            db=self.db,
            name=self.name,
            charset=self.charset)

    def connect(self):
        """建立连接"""

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self._conn = pymysql.connect(**self.setting)
            self.recorder('INFO', '{obj} connect successful ({threadid})'.format(obj=self, threadid=self._conn.server_thread_id[0]))
        except pymysql.Error as e:
            self.recorder('ERROR', '{obj} connect failed [{msg}]'.format(obj=self, msg=e))
            raise MysqlError

        return self

    def ping(self):
        """保持连接"""

        try:
            self.recorder('INFO', '{obj} ping start'.format(obj=self))
            self._conn.ping()
            self.recorder('INFO', '{obj} ping successful'.format(obj=self))
        except pymysql.Error as e:
            self.recorder('WARN', '{obj} ping error [{msg}]'.format(obj=self, msg=e))
            raise MysqlError

    def reconnect(self):
        """重新连接"""

        try:
            self.recorder('WARN', '{obj} reconnect start'.format(obj=self))
            self.connect()
            self._cur = None
            self.recorder('WARN', '{obj} reconnect successful'.format(obj=self))
        except pymysql.Error as e:
            self.recorder('ERROR', '{obj} reconnect error [{msg}]'.format(msg=e))
            raise MysqlError

        return self

    def start_event(self):
        """事务开始"""

        try:
            self._event = True
            self.recorder('INFO', '{obj} start event'.format(obj=self))
            self._conn.begin()
        except pymysql.OperationalError as e:
            self.recorder('WARN', '{obj} event start error [{msg}]'.format(msg=e))
            self.reconnect()
            self.start_event()

    def exec_event(self, sql, args=None):
        """事务执行"""

        if self._event:
            self.recorder('INFO', '{obj} execute event'.format(obj=self))
            return self.query(sql, args)
        else:
            self.recorder('CRITICAL', 'please start event first!')
            raise MysqlError

    def end_event(self):
        """事务结束"""

        if self._event:
            self._event = False
            self.recorder('INFO', '{obj} end event'.format(obj=self))
            self.commit()
        else:
            self.recorder('CRITICAL', 'please start event first! ')
            raise MysqlError

    def query(self, sql, args=None):
        """查询sql"""

        # 执行过程中的重试,只重试一次
        mysql_retry_policy = RetryPolicy(times=1, error=MysqlError)
        return Retry(self, '{obj}'.format(obj=self), self._query, sql, mysql_retry_policy, args).run_sync()

    def _query(self, sql, retry, args):

        if not self._cur:
            self._cur = self._conn.cursor(pymysql.cursors.DictCursor)
        elif self._cur and not self._event:
            self._cur.close()
            self._cur = self._conn.cursor(pymysql.cursors.DictCursor)

        try:
            self._format_sql(sql, args)

            self.recorder('INFO', '{obj} query start ({threadid})\n{sql}'.format(obj=self, threadid=self._conn.server_thread_id[0], sql=self._sql))
            with tool.timing('s', 10) as t:
                self._cur.execute(sql, args)
            self.recorder('INFO',
                          '{obj} query successful\n{sql}\t[{time}]\t[{effect}]'.format(obj=self, sql=self._sql, time=t,
                                                                                       effect=self._cur.rowcount))
        except pymysql.OperationalError as e:
            self.recorder('ERROR', '{obj} mysql has gone away [{msg}]'.format(obj=self, msg=e))
            self.reconnect()
            raise retry
        except (pymysql.IntegrityError, pymysql.ProgrammingError) as e:
            self.recorder('ERROR', '{obj} query error\n{sql}\n[{msg}]'.format(obj=self, sql=self._sql, msg=e))
            raise MysqlError
        except (KeyError, TypeError) as e:
            self.recorder('ERROR',
                          '{obj} sql format error\n{sql}\n{args}\n[{msg}]'.format(obj=self, sql=sql, args=args,
                                                                                  msg=e))
            raise MysqlError

        if not self._event:
            self.commit()

        effect = self._cur.rowcount
        self._cur.close()
        return effect

    def close(self):
        """关闭连接"""

        self.recorder('INFO', '{obj} connection close start'.format(obj=self))
        self._cur.close()
        self._conn.close()
        self.recorder('INFO', '{obj} connection close successful'.format(obj=self))

    def rollback(self):
        """回滚"""

        # TODO:记录本次事务语句
        self._conn.rollback()
        self.recorder('INFO', '{obj} query rollback'.format(obj=self))

    def commit(self):
        """事务提交"""

        self._conn.commit()
        self.recorder('INFO', '{obj} query commit'.format(obj=self))


class AsynMysql(Mysql):
    """异步mysql组件"""

    def __reduce__(self):
        return AsynMysql, (self.setting,)

    def __str__(self):
        return '<AsynMysql|{name}|{id} {host} {port} {user} {db} {charset}>'.format(
            id=id(self),
            host=self.host,
            port=self.port,
            user=self.user,
            db=self.db,
            name=self.name,
            charset=self.charset)

    @coroutine
    def connect(self):
        """建立连接"""

        try:
            self.recorder('INFO', '{obj} connect start'.format(obj=self))
            self._conn = yield tornado_mysql.connect(**self.setting)
            self.recorder('INFO', '{obj} connect successful ({threadid})'.format(obj=self, threadid=self._conn.server_thread_id[0]))
        except (tornado_mysql.Error, tornado_mysql.OperationalError) as e:
            self.recorder('ERROR', '{obj} connect error [{msg}]'.format(obj=self, msg=e))
            raise MysqlError

        raise Return(self)

    @coroutine
    def ping(self):
        """维持连接"""

        try:
            self.recorder('WARN', '{obj} ping start'.format(obj=self))
            yield self._conn.ping()
            self.recorder('WARN', '{obj} ping successful'.format(obj=self))
        except tornado_mysql.Error as e:
            self.recorder('WARN', '{obj} ping error [{msg}]'.format(obj=self, msg=e))
            raise MysqlError

    @coroutine
    def reconnect(self):
        """重新连接"""

        try:
            self.recorder('WARN', '{obj} reconnect start'.format(obj=self))
            yield self.connect()
            self._cur = None
            self.recorder('WARN', '{obj} reconnect successful'.format(obj=self))
        except tornado_mysql.Error as e:
            self.recorder('ERROR', '{obj} reconnect error [{msg}]'.format(obj=self, msg=e))
            raise MysqlError

        raise Return(self)

    @coroutine
    def start_event(self):
        """事务开始"""

        try:
            self._event = True
            self.recorder('INFO', '{obj} start event'.format(obj=self))
            yield self._conn.begin()
        except tornado_mysql.OperationalError as e:
            self.recorder('WARN', '{obj} event start error [{msg}]'.format(obj=self, msg=e))
            yield self.reconnect()
            yield self.start_event()

    @coroutine
    def exec_event(self, sql, args=None):
        """事务执行"""

        if self._event:
            self.recorder('INFO', '{obj} execute event'.format(obj=self))
            rows = yield self.query(sql, args)
            raise Return(rows)
        else:
            self.recorder('CRITICAL', 'please start event first!')
            raise MysqlError

    @coroutine
    def end_event(self):
        """事务结束"""

        if self._event:
            self._event = False
            self.recorder('INFO', '{obj} end event'.format(obj=self))
            yield self.commit()
        else:
            self.recorder('CRITICAL', 'please start event first! ')
            raise MysqlError

    @coroutine
    def query(self, sql, args=None):
        """查询sql"""

        mysql_retry_policy = RetryPolicy(times=1, error=MysqlError)
        effect = yield Retry(self, '{obj}'.format(obj=self), self._query, sql, mysql_retry_policy, args).run_asyn()
        raise Return(effect)

    @coroutine
    def _query(self, sql, retry, args=None):
        if not self._cur:
            self._cur = self._conn.cursor(tornado_mysql.cursors.DictCursor)
        elif self._cur and not self._event:
            self._cur.close()
            self._cur = self._conn.cursor(tornado_mysql.cursors.DictCursor)

        try:
            self._format_sql(sql, args)

            self.recorder('INFO', '{obj} query start ({threadid})\n{sql}'.format(obj=self,
                          threadid=self._conn.server_thread_id[0], sql=sql))
            with tool.timing('ms', 10) as t:
                yield self._cur.execute(sql, args)
            self.recorder('INFO',
                          '{obj} query successful\n{sql}\t[{time}]\t[{effect}]'.format(obj=self, sql=sql, time=t,
                                                                                       effect=self._cur.rowcount))
        except (tornado_mysql.OperationalError, tornado_mysql.InterfaceError, iostream.StreamClosedError) as e:
            self.recorder('ERROR', '{obj} mysql has gone away [{msg}]'.format(obj=self, msg=e))
            yield self.reconnect()
            raise retry
        except (tornado_mysql.IntegrityError, tornado_mysql.ProgrammingError) as e:
            self.recorder('ERROR', '{obj} query error\n{sql}\n[{msg}]'.format(obj=self, sql=sql, msg=e))
            raise MysqlError
        except (KeyError, TypeError) as e:
            self.recorder('ERROR',
                          '{obj} sql format error\n{sql}\n{args}\n[{msg}]'.format(obj=self, sql=sql, args=args,
                                                                                  msg=e))
            raise MysqlError

        if not self._event:
            yield self.commit()

        effect = self._cur.rowcount
        yield self._cur.close()
        raise Return(effect)

    @coroutine
    def rollback(self):
        """回滚"""

        yield self._conn.rollback()

    @coroutine
    def commit(self):
        """提交操作"""

        yield self._conn.commit()

    @coroutine
    def close(self):
        """关闭连接"""

        yield self._cur.close()
        yield self._conn.close()



# coding:utf8


from fastweb.component.db.mysql import (SyncMysql, AsynMysql)
from fastweb.component.db.rds import (AsynRedis, SyncRedis)
from fastweb.component.db.mongo import (AsynMongo, SyncMongo)
from fastweb.component.rpc.tft import (AsynTftRpc, SyncTftRpc)


# 默认连接池大小
DEFAULT_POOL_SIZE = 10

# 同步组件
SYNC_CONN_COMPONENTS = [('mysql', SyncMysql, DEFAULT_POOL_SIZE),
                        ('redis', SyncRedis, DEFAULT_POOL_SIZE),
                        ('mongo', SyncMongo, DEFAULT_POOL_SIZE),
                        ('tftrpc', SyncTftRpc, DEFAULT_POOL_SIZE)]

# 异步组件
ASYN_CONN_COMPONENTS = [('mysql', AsynMysql, DEFAULT_POOL_SIZE),
                        ('redis', AsynRedis, DEFAULT_POOL_SIZE),
                        ('mongo', AsynMongo, DEFAULT_POOL_SIZE),
                        ('tftrpc', AsynTftRpc, DEFAULT_POOL_SIZE)]



# coding:utf8

from fastweb.loader import app
from fastweb.service import start_service_server


if __name__ == '__main__':
    app.load_recorder('service.log', system_level='DEBUG')
    app.load_component(layout='service', backend='ini', path='fastweb/test/config/service.ini')
    start_service_server()



# coding:utf8


ERRCODE = {
    'SUC': {'code': 0, 'message': 'success'},
    'ARG': {'code': 1001, 'message': 'invalid arguments'},
    'SVR': {'code': 2001, 'message': 'server error'},
    'TOKEN': {'code': 3995, 'message': 'token invalid'},
    'KEY': {'code': 4001, 'message': 'key not exist'},
    'EXT': {'code': 4002, 'message': 'key exist'},
    'PWD': {'code': 4003, 'message': 'password wrong'},
    'FMT': {'code': 4004, 'message': 'format error'},
    'FT': {'code': 4005, 'message': 'upload file type not support'},
}



# coding:utf8


from fastweb import app
from fastweb.task import start_task_worker


class TestWorker(object):
    @staticmethod
    def test_start_worker():
        app.load_component(layout='task', backend='ini', path='config/worker.ini')
        start_task_worker()

if __name__ == '__main__':
    TestWorker.test_start_worker()




# coding:utf8


from setuptools import setup, find_packages

kwargs = {}

version = '1.4.1.21_1'

with open('README.rst') as f:
        kwargs['long_description'] = f.read()

install_requires = []
with open('requirements.txt') as f:
    for require in f:
        install_requires.append(require[:-1])

kwargs['install_requires'] = install_requires

setup(
    name='fastweb',
    version=version,
    packages=find_packages(),
    package_data={'fastweb': ['setting/default_logging.yaml']},
    scripts=['requirements.txt'],
    entry_points={
        'console_scripts': [
            'fasthrift = fastweb.command.service.thrift:gen_thrift_auxiliary',
            'fast = fastweb.command.fast:main'
        ],
    },
    author='Bslience',
    description="FastWeb is a Python fast-building web frame refered by Tornado, Celery, Thrift",
    **kwargs
)



# coding:utf8


from fastweb import coroutine, ioloop
import tornado_mysql
from fastweb.util.tool import timing


@coroutine
def test():
    conn = yield tornado_mysql.connect(**{'host': 'localhost', 'port': 3306, 'user': 'root', 'password': '1a2s3dqwe'})
    with timing('ms', 10) as ti:
        for _ in xrange(30):
            cur = conn.cursor(tornado_mysql.cursors.DictCursor)
            with timing('ms', 10) as t:
                yield cur.execute('SELECT * FROM mysql.user;')
            print t

    print 'total:'
    print ti

ioloop.IOLoop.current().run_sync(test)



# coding:utf8
import logging
logging.basicConfig(level=logging.INFO)

import sys
import glob
sys.path.append('gen-py.tornado')

from HelloService import HelloService
from HelloService.ttypes import *

from thrift import TTornado
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol
from thrift.protocol import TMultiplexedProtocol

from tornado import gen
from tornado import ioloop


@gen.coroutine
def communicate():
    # create client
    transport = TTornado.TTornadoStreamTransport('localhost', 9999)
    # open the transport, bail on error
    try:
        yield transport.open()
        print('Transport is opened')
    except TTransport.TTransportException as ex:
        logging.error(ex)
        raise gen.Return()

    protocol = TBinaryProtocol.TBinaryProtocolFactory()
    #pfactory = TMultiplexedProtocol.TMultiplexedProtocol(protocol, 'hello')
    client = HelloService.Client(transport, protocol)

    # ping
    yield client.sayHello()
    print("ping()")

    client._transport.close()
    raise gen.Return()


def main():
    # create an ioloop, do the above, then stop
    import time
    start = time.time()
    for _ in xrange(10000):
        ioloop.IOLoop.current().run_sync(communicate)
    end = time.time()
    print end - start
if __name__ == "__main__":
    main()



# coding:utf8


import unittest

import init
from fastweb.util.configuration import Configuration


class ConfigurationTest(unittest.TestCase):

    def runTest(self):
        configuration = Configuration('ini', path='config/config.ini')
        print configuration.configs
        print configuration.get_components('mysql')

if __name__ == '__main__':
    unittest.main()



# coding:utf8

from fastweb.accesspoint import ioloop
from fastweb.component.rpc.tft import AsynTftRpc, SyncTftRpc

sync_setting = {'host': 'localhost', 'port': 7777, 'thrift_module': 'fastweb.test.fastweb_thrift_sync.HelloService.HelloService'}
asyn_setting = {'host': 'localhost', 'port': 7777, 'thrift_module': 'fastweb.test.fastweb_thrift_async.HelloService.HelloService'}


class TestSyncThrift(object):
    def test_connect(self):
        rpc = SyncTftRpc(sync_setting).set_name('test sync thrift')
        rpc.connect()
        rpc.close()

    def test_call(self):
        rpc = SyncTftRpc(sync_setting).set_name('test sync thrift')
        rpc.connect()
        assert rpc.sayHello() == 'hello'
        rpc.close()


class TestAsynThrift(object):
    def test_connect(self):
        rpc = AsynTftRpc(asyn_setting).set_name('test asyn thrift')
        ioloop.IOLoop.current().run_sync(rpc.connect)
        rpc.close()

    def test_call(self):
        rpc = AsynTftRpc(asyn_setting).set_name('test asyn thrift')
        ioloop.IOLoop.current().run_sync(rpc.connect)
        assert ioloop.IOLoop.current().run_sync(rpc.sayHello) == 'hello'
        rpc.close()




DEFAULT_LOGGING_SETTING =\
{
    "loggers": {
        "application_recorder": {
            "level": "DEBUG",
            "propagate": False,
            "handlers": [
                "application_file_time_handler"
            ]
        },
        "system_recorder": {
            "level": "DEBUG",
            "propagate": False,
            "handlers": [
                "console_handler",
                "system_file_size_handler"
            ]
        }
    },
    "version": 1,
    "formatters": {
        "application_formatter": {
            "format": "[%(requestid)s] [%(levelname)s] [%(asctime)s] [%(process)d:%(thread)d]\n%(message)s"
        },
        "system_formatter": {
            "format": "[%(levelname)s] [%(asctime)s] [%(process)d:%(thread)d]\n%(message)s"
        }
    },
    "disable_existing_loggers": False,
    "handlers": {
        "console_handler": {
            "formatter": "system_formatter",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout"
        },
        "application_file_time_handler": {
            "formatter": "application_formatter",
            "backupCount": 20,
            "encoding": "utf8",
            "interval": 1,
            "when": "D",
            "class": "logging.handlers.TimedRotatingFileHandler",
        },
        "system_file_size_handler": {
            "formatter": "system_formatter",
            "backupCount": 20,
            "encoding": "utf8",
            "maxBytes": 10485760,
            "class": "logging.handlers.RotatingFileHandler",
        }
    }
}